<?php
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

if ( class_exists( 'Tribe__Events__Ical_Importer__Feed_Parser' ) ) {
	return;
}


/**
 * Tribe Events iCal Importer Feed Parser
 *
 * Use this class to retrieve all events from a iCal file or url
 * Will turn the iCal feed into an array of objects
 *
 * @param string $ical - file location or url
 *
 * @uses    new Tribe__Events__Ical_Importer__Feed_Parser( $ical )->get_events();
 *
 * @package Tribe__Events__Ical_Importer__Main
 *
 */
class Tribe__Events__Ical_Importer__Feed_Parser extends vcalendar {

	/**
	 * Events
	 *
	 * The parsed events ready for importing
	 *
	 * @var array
	 */
	public $events = array();

	/**
	 * Raw Events
	 *
	 * The events parsed by vcalendar before conversion
	 *
	 * @var array
	 */
	public $raw_events = array();

	/**
	 * Source
	 *
	 * The source of the iCal set during parse
	 *
	 * @var string
	 */
	public $source;

	/**
	 * Retrieved Geo Coordinates
	 *
	 * Keeps track of coordinates fetched from google to prevent duplicate calls
	 *
	 * @var array
	 */
	public static $retrieved_geo_coordinates = array();

	/**
	 * Matched Search Criteria
	 *
	 * Keeps track of which parts of search returned events
	 * For use when no events make it through the filtering
	 *
	 * @var array
	 */
	public $matched_search_criteria = array(
		'startDate' => false,
		'keywords'  => false,
		'location'  => false,
		'category'  => false,
	);

	/**
	 * Used Uids
	 *
	 * Keeps track of the uids so we don't double up due to funky data
	 *
	 * @var array
	 */
	public $used_uids = array();

	/**
	 * Previous Parents
	 *
	 * Keeps track of the uids which were parents before to prevent import loops
	 *
	 * @var array
	 */
	public $previous_parents = array();

	/**
	 * Lat
	 *
	 * For the filter by location and distance
	 *
	 * @var float
	 */
	public $lat;

	/**
	 * Long
	 *
	 * For the filter by location and distance
	 *
	 * @var float
	 */
	public $long;

	/**
	 * Args
	 *
	 * The defaults args merged with whatever args are sent
	 *
	 * @var array
	 */
	public $args = array(
		'start'    => 'now',
		'end'      => '+2 year',
		'count'    => '',
		'keywords' => '',
		'location' => '',
		'category' => '', //not included in very many ical feeds
		'manual'   => false,
	);

	/**
	 * @var Tribe__Events__Ical_Importer__Utils
	 */	
	protected $utils;

	/**
	 * Criteria
	 *
	 * Used internal to keep track of event filtering criteria
	 * Automatically set based on what type of $this->selectComponents we run
	 *
	 * @var array
	 */
	private $criteria = array(
		'start'    => false,
		'end'      => false,
		'category' => false,
	);

	/**
	 * Timezone
	 *
	 * Stores the global timezone set for the iCal file (X-WR-TIMEZONE)
	 *
	 * @var string
	 */
	public $timezone;

	/**
	 * Takes care of all parsing and puts final events into $this->events
	 *
	 * @param string $ical file or url
	 * @param array  $args = [
	 *      'start'    => string,       // start date
	 *      'end'      => string,       // end date
	 *      'count'    => int,          // limit number of events
	 *      'keywords' => array|string, // words an event must include
	 *      'location' => string,       // location of event
	 *      'radius'   => int,          // used in conjunction with location for how far
	 *      'category' => array|string, // event categories - not included in very many ical feeds
	 *      'manual'   => bool          // if we are doing a search or manual import
	 * ]
	 * @param null|Tribe__Events__Ical_Importer__Utils $utils
	 */
	public function __construct( $ical, $args = array(), Tribe__Events__Ical_Importer__Utils $utils = null ) {
		$this->utils = ! empty( $utils ) ? $utils : Tribe__Events__Ical_Importer__Utils::instance();
		$this->deactivate_full_recurrence_support();
		$this->args = wp_parse_args( $args, $this->args );
		$is_remote  = false;

		parent::__construct( array(
			'unique_id' => 'Tribe__Events__Ical_Importer__Feed_Parser' . microtime( true ),
		) );
		$ical = str_replace( 'webcal://', 'http://', $ical );

		if ( file_exists( $ical ) ) {
			$content = file_get_contents( $ical );
		} else {
			$is_remote = true;
			$content   = $this->get_remote_content( $ical );
		}

		// Content could not be loaded: an appropriate error should already have been logged
		if ( false === $content ) {
			return;
		}

		$calendar = $this->parse( $content );

		// Report errors according to whether the source was local or remote
		if ( ! $calendar && $is_remote ) {
			$this->events = new WP_Error( 1, __( 'No iCal data was found at this URL.', 'tribe-events-ical-importer' ) );
		} elseif ( ! $calendar ) {
			$this->events = new WP_Error( 1, __( 'Not a valid .ics file', 'tribe-events-ical-importer' ) );
		}

		// If we did hit an error, do not go any further
		if ( ! $calendar ) {
			return;
		}

		// Set the source to be used later
		if ( ! empty( $this->source[1] ) ) {
			$this->source = $this->source[1];
		} else {
			$this->source = $this->getProperty( 'X-WR-CALNAME' );
		}

		// Turn events into usable objects and apply args and filters
		$this->parse_events();
	}

	/**
	 * Attempts to load the content at the provided URL via the WP HTTP API.
	 *
	 * @param  string $url
	 *
	 * @return mixed bool|string
	 */
	protected function get_remote_content( $url ) {
		$url = str_replace( 'webcal://', 'http://', $url );

		/**
		 * Length of time to wait when initially connecting to a remote iCal feed before
		 * abandoning the attempt.
		 *
		 * @var int $timeout_in_seconds
		 */
		$timeout_in_seconds = (int) apply_filters( 'tribe_events_ical_importer_connection_timeout', 5 );

		$response = null;

		foreach ( $this->utils->headers() as $headers ) {
		$request_args = array(
			'timeout'     => $timeout_in_seconds,
			'sslverify'   => false,
			'redirection' => 4,
			'method'      => 'GET',
			'headers'     => $headers,
		);

		$response = wp_remote_get( $url, $request_args );

			if ( ! is_wp_error( $response ) ) {
				break;
			}

		if ( is_wp_error( $response ) ) {
			// let's try again with `sslverify` set to `true`
			$request_args['sslverify'] = true;
				$response                  = wp_remote_head( $url, $request_args );

				if ( ! is_wp_error( $response ) ) {
					break;
				}
			}
		}

		if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) != 200 ) {
			$this->events = new WP_Error( 1, __( 'Unable to retrieve content from the provided URL.', 'tribe-events-ical-importer' ) );

			return false;
		}

		return $response['body'];
	}

	/**
	 * Get Events
	 *
	 * Return the parsed events in an importable form
	 *
	 * @uses $this->events;
	 *
	 * @return array
	 *
	 */
	public function get_events() {
		return $this->events;
	}

	/**
	 * Check If Event Is In Radius
	 *
	 * Checks to see if an event is within the specified radius of a location
	 *
	 * @uses $this->lat
	 * @uses $this->long
	 *
	 * @param string $geo
	 * @param int    $radius
	 *
	 * @return boolean
	 */
	public function check_if_event_is_in_radius( $geo, $radius ) {
		if ( empty( $this->lat ) || empty( $this->long ) ) {
			return false;
		}

		list( $lat, $long ) = explode( ';', $geo );

		if ( (float) $this->lat == (float) $lat && (float) $this->long == (float) $long ) {
			return true;
		}

		//take our km and convert back to miles
		$radius = $radius * 0.621371;

		$longitude = (float) $this->long;
		$latitude  = (float) $this->lat;

		$minLng = $longitude - $radius / abs( cos( deg2rad( $latitude ) ) * 69 );
		$maxLng = $longitude + $radius / abs( cos( deg2rad( $latitude ) ) * 69 );
		$minLat = $latitude - ( $radius / 69 );
		$maxLat = $latitude + ( $radius / 69 );


		if ( ( (float) $lat > (float) $minLat ) && ( (float) $lat < (float) $maxLat ) ) {

			if ( ( (float) $long > (float) $minLng ) && ( (float) $long < (float) $maxLng ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Lat Long Set
	 *
	 * Sets the lat and long class var based on an address
	 *
	 * @param string $location - address
	 *
	 * @return void
	 *
	 */
	public function lat_long_set( $location ) {
		$geo = wp_remote_get( 'http://maps.googleapis.com/maps/api/geocode/json?address=' . urlencode( $location ) );

		if ( ! empty( $geo['body'] ) ) {
			$geo = json_decode( $geo['body'] );
		}

		if ( ! empty( $geo->results[0]->geometry->location->lat ) ) {
			$this->lat = $geo->results[0]->geometry->location->lat;
		}

		if ( ! empty( $geo->results[0]->geometry->location->lng ) ) {
			$this->long = $geo->results[0]->geometry->location->lng;
		}
	}

	/**
	 * Parse Events
	 *
	 * Turn the vcalendar object into importable events
	 * All the args based filtering occurs here
	 * Filtered events are stored in $this->events
	 *
	 * @uses $this->events
	 *
	 * @return void
	 *
	 */
	public function parse_events() {
		$events_count = 0;

		/**
		 * @todo revisit this variables to check which ones are required and it's defaults values
		 */
		$defaults = array(
			'start'    => 'now',
			'end'      => '+2 year',
			'count'    => '',
			'keywords' => '',
			'location' => '',
			'category' => '', //not included in very many ical feeds
			'manual'   => false,
			'paged'    => 1,
			'url'      => '',
			'radius'   => '',
			'schedule' => '',
			'action'   => '',
		);

		// Prevents Notices
		$this->args = wp_parse_args( $this->args, $defaults );

		$start    = $this->args['start'];
		$end      = $this->args['end'];
		$count    = $this->args['count'];
		$keywords = $this->args['keywords'];
		$location = $this->args['location'];
		$category = $this->args['category'];
		$manual   = $this->args['manual'];
		$url      = $this->args['url'];
		$radius   = $this->args['radius'];
		$schedule = $this->args['schedule'];
		$paged    = $this->args['paged'];
		$action   = $this->args['action'];

		$this->timezone = $this->utils->parse_timezone( $this );

		$this->raw_events = $this->get_raw_events_by_range();

		$this->criteria['start'] = strtotime( $this->args['start'] );
		$this->criteria['end']   = strtotime( $this->args['end'] );

		if ( ! empty( $location ) ) {
			$this->lat_long_set( $location );
			if ( empty( $radius ) ) {
				$radius = Tribe__Events__Importer__Options::getOption( 'geoloc_default_geofence', 25 );
			}
		}

		//per year
		foreach ( $this->raw_events as $year => $year_arr ) {
			if ( strtotime( "$year-12-31" ) < $this->criteria['start'] ) {
				continue;
			}

			if ( strtotime( "$year-01-01" ) > $this->criteria['end'] ) {
				continue;
			}

			//per month
			foreach ( $year_arr as $month => $month_arr ) {
				$mm = str_pad( $month, 2, '0', STR_PAD_LEFT );
				$d = new DateTime( "{$year}-{$mm}-01" );
				$d->setDate( $year, $mm, $d->format( 't' ) );

				if ( $d->format( 'U' ) < $this->criteria['start'] ) {
					continue;
				}

				if ( strtotime( "{$year}-{$mm}-01" ) > $this->criteria['end'] ) {
					continue;
				}

				//per day
				foreach ( $month_arr as $day => $day_arr ) {
					$dd = str_pad( $day, 2, '0', STR_PAD_LEFT );

					if ( strtotime( "{$year}-{$mm}-{$dd} 23:59:59" ) < $this->criteria['start'] ) {
						continue;
					}

					if ( strtotime( "{$year}-{$mm}-{$dd}" ) > $this->criteria['end'] ) {
						continue;
					}

					$this->matched_search_criteria['startDate'] = true;

					//each event
					foreach ( $day_arr as $event ) {
						//check count
						if ( ! empty( $count ) && $events_count > $count ) {
							continue;
						}

						//check categories
						if ( ! empty( $category ) ) {
							$categories = array();
							while ( $cat = $event->getProperty( 'CATEGORIES' ) ) {
								$categories[] = $cat;
							}

							if ( empty( $categories ) ) {
								continue;
							}

							if ( ! in_array( strtoupper( $category ), $categories ) ) {
								continue;
							}

							$this->matched_search_criteria['category'] = true;
						}//end if


						//check location
						if ( ! empty( $location ) ) {
							$geo = $this->get_geo_coordinates( $event );
							if ( empty( $geo ) ) {
								continue;
							}

							if ( ! $this->check_if_event_is_in_radius( $geo, $radius ) ) {
								continue;
							}

							$this->matched_search_criteria['location'] = true;
						}//end if


						//check for keywords
						if ( ! empty( $keywords ) ) {
							$fail = false;

							foreach ( (array) $keywords as $word ) {
								if ( false === stripos( $event->getProperty( 'SUMMARY' ), $word ) && false === stripos( $event->getProperty( 'DESCRIPTION' ), $word ) ) {
									$fail = true;
									break;
								}
							}//end foreach

							if ( $fail ) {
								continue;
							}

							$this->matched_search_criteria['keywords'] = true;
						}//end if

						//made it past the general criteria
						$this->convert_vcal_event_to_importable_event( $event );

						$events_count ++;
					} //end foreach
				} //end foreach
			} //end foreach
		} //end foreach

		//if did a location query we have some new location data to cache
		if ( ! empty( $location ) ) {
			set_transient( 'tribe-events-ical-importer-geo', self::$retrieved_geo_coordinates );
		}
	}

	/**
	 * Convert DTSTART/DTEND and X-CURRENT-DTSTART/X-CURRENT-DTEND to date arrays
	 *
	 * @param string $dtstart           DTSTART property value
	 * @param string $dtend             DTEND property value
	 * @param string $x_current_dtstart X-CURRENT-DTSTART property value
	 * @param string $x_current_dtend   X-CURRENT-DTEND property value
	 *
	 * @return array
	 */
	public function convert_dates_to_date_arrays( $dtstart, $dtend, $x_current_dtstart, $x_current_dtend ) {
		$dtstart_parts = iCalPropertyFactory::strdate2arr( $dtstart );
		$dtend_parts   = iCalPropertyFactory::strdate2arr( $dtend );

		// if there isn't a dtstart, fallback to x-current-dtstart and x-current-dtend
		if ( ! empty( $x_current_dtstart[1] ) && $dtstart != $x_current_dtstart[1] ) {
			$start = iCalPropertyFactory::strdate2arr( $x_current_dtstart[1] );
			$end   = iCalPropertyFactory::strdate2arr( $x_current_dtend[1] );

			$dtend_timestamp = strtotime( "{$dtend_parts['year']}-{$dtend_parts['month']}-{$dtend_parts['day']}" );
			$xend_timestamp  = strtotime( "{$end['year']}-{$end['month']}-{$end['day']}" );

			// if the DTEND date is greater than the X-CURRENT-DTEND, we're looking at a multi-day event.
			// Convert the end date to span the full set of days
			if ( $dtend_timestamp > $xend_timestamp ) {
				$end['year']  = $dtend_parts['year'];
				$end['month'] = $dtend_parts['month'];
				$end['day']   = $dtend_parts['day'];
			}

			// Use the start time from dtstart
			if ( ! empty( $dtstart_parts['hour'] ) ) {
				$start['hour'] = $dtstart_parts['hour'];
				$start['min']  = $dtstart_parts['min'];
				$start['sec']  = $dtstart_parts['sec'];
			}

			// Use the end time from dtend
			if ( ! empty( $dtend_parts['hour'] ) ) {
				$end['hour'] = $dtend_parts['hour'];
				$end['min']  = $dtend_parts['min'];
				$end['sec']  = $dtend_parts['sec'];
			}
		} else {
			$start = $dtstart_parts;
			$end   = $dtend_parts;
		}

		return array(
			'start' => $start,
			'end'   => $end,
		);
	}

	/**
	 * Convert Vcal Event To Importable Event
	 *
	 * Turns a vcalendar event in an import ready object
	 * Adds the converted event to $this->events
	 *
	 * @param iCalVEVENT $event
	 *
	 * @uses $this->events
	 *
	 * @return void
	 *
	 */
	public function convert_vcal_event_to_importable_event( iCalVEVENT $event ) {
		//child events of a recurring event
		$child_event = $event->getProperty( 'X-RECURRENCE' );
		
		$uid = $this->utils->generate_uid_for_event( $event );

		//if we are doing a full import and have PRO installed child events will cause crazy loops of events
		if ( ! $this->manual() && $this->support_recurrence() ) {
			if ( in_array( $uid, $this->previous_parents ) ) {
				 return;
			}

			$this->previous_parents[] = $uid;

		//if PRO is not installed all we care about is if this event is the first in a series
		} else {
			//if we have not seen this uid before this must not be a child
			if ( ! in_array( $uid, $this->previous_parents ) ) {
				$child_event              = false;
				$this->previous_parents[] = $uid;
			}
		}

		// get a few timezones we care about
		$system_timezone = date_default_timezone_get();
		$wp_timezone     = Tribe__Events__Timezones::wp_timezone_string();

		$start_data = $event->getProperty( 'DTSTART', false, true );
		$start_str  = $start_data['value'];
		$start      = iCalPropertyFactory::strdate2arr( $start_str );
		$start_time = strtotime( $start_str );

		$end_str  = $event->getProperty( 'DTEND' );
		$end      = iCalPropertyFactory::strdate2arr( $end_str );
		$end_time = strtotime( $end_str );
		// Let's determine if this is an all-day event, whether it's spanning on multiple days or not	

		$is_all_day = ! isset( $start['hour'] );
		
		// let's determine what timezone the event is represented as
		$timezone = null;
		if ( ! empty( $start['tz'] ) ) {
			// if the start date is in UTC time (with the Z at the end) then set the timezone as UTC
			// BUT ONLY IF there isn't an X-WR-TIMEZONE because the iCal importer will convert to the
			// X-WR-TIMEZONE for us when if x_start_time is set
			$timezone = 'UTC';
		} elseif ( preg_match( '/TZID=([^;]+)/', $start_data['params'], $matches ) ) {
			$is_valid = $this->utils->is_valid_php_timezone( $matches[1] );
			if ( $is_valid ) {
				$timezone = $matches[1];
			} else {
				$timezone = $this->timezone;
			}
		} elseif ( ! empty( $this->timezone ) ) {
			// if there is a global timezone in the iCal file, use that
			$timezone = $this->timezone;
		}

		// Set time ID
		$time_id      = $start_str;
		$is_multi_day = ( $start['day'] != $end['day'] );

		// Fetch the generated X-CURRENT-DTSTART|DTEND which is useful primarily for repeating events, but is
		// available for any. If it isn't delcared, it is generated. The PROBLEM with the result is that
		// it ALWAYS assumes that the datetime string that is provided (via the feed and within iCalCreator)
		// is relative to the current system timezone regardless of whether or not the feed is representing
		// it as Zulu time or not. We'll use the maybe_convert_from_system_timezone to correct that
		$x_start_str  = (array) $event->getProperty( 'X-CURRENT-DTSTART' );
		$x_start_str  = $this->maybe_convert_from_system_timezone( end( $x_start_str ), $system_timezone, $timezone );
		$x_start_time = strtotime( $x_start_str );

		$x_end_str  = (array) $event->getProperty( 'X-CURRENT-DTEND' );
		$x_end_str  = $this->maybe_convert_from_system_timezone( end( $x_end_str ), $system_timezone, $timezone );
		$x_end_time = strtotime( $x_end_str );

		//if this event has an X-CURRENT-DTSTART the correct date will be X-CURRENT-DTSTART
		if ( ! empty( $x_start_time ) && ! empty( $x_end_time ) ) {
			$start = iCalPropertyFactory::strdate2arr( $x_start_str );

			// iCalCreator will generate the X-CURRENT-DTEND property if not set: sometimes that's wrong.
			// Let's sanity check it against the DTEND.
			$start_time_diff = $x_start_time - $start_time;
			if ( ( $x_end_time - $end_time ) === $start_time_diff ) {
				$end = iCalPropertyFactory::strdate2arr( $x_end_str );
			}

			// Set time ID
			$time_id = $x_start_str;
		}

		//child events need a id unique to event not just series
		if ( ! empty( $child_event ) && ! $this->args['manual'] ) {
			$uid = $event->getProperty( 'UID' ) . '-' . $time_id;
		}

		//there should be no reason we have duplicates
		if ( in_array( $uid, $this->used_uids ) ) {
			return;
		}

		$this->used_uids[] = $uid;

		// All day event
		if ( $is_all_day ) {
			$all_day       = true;
			$start['hour'] = 0;
			$start['min']  = 1;
			$end['hour']   = 23;
			$end['min']    = 59;
			/**
			 * all day events come in as ending the next day
			 * set the end to one day less so multiple day events still work and single
			 * events only span one day
			 */
			if ( $start['day'] != $end['day'] ) {
				$end['day'] = $end['day'] - 1;
			}
		} else {
			$all_day = false;
		}

		/**
		 * If we don't have time, just set it to zero which is a safer call
		 * This will prevent strtotime not understanding the pattern
		 */
		if ( empty( $start['hour'] ) ) {
			$start['hour'] = '0';
		}

		if ( empty( $start['min'] ) ) {
			$start['min'] = '0';
		}

		if ( empty( $start['sec'] ) ) {
			$start['sec'] = '0';
		}

		if ( empty( $end['hour'] ) ) {
			$end['hour'] = '0';
		}

		if ( empty( $end['min'] ) ) {
			$end['min'] = '0';
		}

		if ( empty( $end['sec'] ) ) {
			$end['sec'] = '0';
		}

		$google_maps_enabled = Tribe__Events__Importer__Options::getOption( 'ical_import_enable_GoogleMaps', false );
		
		$post                = array(
			'post_title'       => tribe_ical_replace_linebreaks_with_spaces( $event->getProperty( 'SUMMARY' ) ),
			'post_type'        => Tribe__Events__Main::POSTTYPE,
			'post_content'     => tribe_ical_inject_linebreaks( $event->getProperty( 'DESCRIPTION' ) ),
			'EventStartDate'   => sprintf( "%'.04d-%'.02d-%'.02d", $start['year'], $start['month'], $start['day'] ),
			'EventEndDate'     => sprintf( "%'.04d-%'.02d-%'.02d", $end['year'], $end['month'], $end['day'] ),
			'EventStartHour'   => $start['hour'],
			'EventEndHour'     => $end['hour'],
			'EventStartMinute' => $start['min'],
			'EventEndMinute'   => $end['min'],
			'EventURL'         => $event->getProperty( 'URL' ),
			'Venue'            => $this->get_venue_data_array( $event ),
			'_uid'             => $uid,
			'dev_start'        => $start,
			'dev_end'          => $end,
			'EventShowMap'     => $google_maps_enabled,
			'EventShowMapLing' => $google_maps_enabled,
		);

		// set the EventAllDay property on the event
		if ( $is_all_day ) {
			$post['EventAllDay'] = 'yes';
		}

		// convert the event's timezone to the WP timezone
		if ( ! empty( $timezone ) && 'UTC' === $timezone ) {
			$post                  = $this->store_date_as_utc( $post );
			$post                  = $this->generate_timezone_date_from_utc( $post, $wp_timezone );
			$post['EventTimezone'] = $wp_timezone;
		} elseif ( ! empty( $timezone ) ) {
			$post                  = $this->generate_utc_from_timezone_date( $post, $timezone );
			$post                  = $this->generate_timezone_date_from_utc( $post, $wp_timezone );
			$post['EventTimezone'] = $wp_timezone;
		} else {
			$post                  = $this->generate_utc_from_timezone_date( $post, $wp_timezone );
			$post['EventTimezone'] = $wp_timezone;
		}

		// Add geoloc coordinates if we can find them
		$geo = self::get_geo_coordinates( $event );
		if ( ! empty( $geo ) ) {
			list( $lat, $lng ) = explode( ';', $geo );
			$post['Venue']['OverwriteCoords'] = 1;
			$post['Venue']['Lat']             = $lat;
			$post['Venue']['Lng']             = $lng;
		}

		//if we are child event set the parent for referrence
		//$this->manual is no good because non Pro will use this
		if ( ! empty( $child_event ) && ! $this->args['manual'] ) {
			$post['parent_uid'] = $event->getProperty( 'UID' );
		}

		// If this is a recurring event we need to setup the appropriate recurrence meta expected by PRO or
		// else it will not be listed as a recurring event after import. We don't do this for manually imported
		// events (one-time imports) since it may be that only a small subset of events will be imported (and so
		// they will not be representative of the RRULE)
		$rrule = $event->getProperty( 'RRULE' );
		if ( ! empty( $rrule ) && ! $this->args['manual'] ) {
			$post['recurrence'] = $rrule;
		}

		//the organizer
		$organizer = $event->getProperty( 'ORGANIZER', false, true, true );

		if ( ! empty( $organizer ) ) {
			$params = wp_parse_args( str_replace( ';', '&', $organizer['params'] ) );
			foreach ( $params as $k => $param ) {
				if ( $k == 'CN' ) {
					$post['Organizer']['Organizer'] = preg_replace( '/^"(.*)"$/', '\1', $param );
				} else {
					if ( ! empty( $param ) ) {
						$post['Organizer'][ $k ] = $param;
					}
				}
			}

			if ( empty( $post['Organizer']['Organizer'] ) ) {
				$post['Organizer']['Organizer'] = str_replace( 'MAILTO:', '', $organizer['value'] );
			} elseif ( empty( $post['Organizer']['Email'] ) && ! empty( $organizer['value'] ) ) {
				$post['Organizer']['Email'] = str_replace( 'MAILTO:', '', $organizer['value'] );
			}
		}

		//if we have categories - rare
		$categories = array();
		while ( $cat = $event->getProperty( 'CATEGORIES' ) ) {
			$categories[] = $cat;
		}
		if ( ! empty( $categories ) ) {
			foreach ( $categories as $cat ) {
				$cat                  = ucwords( strtolower( $cat ) );
				$post['categories'][] = $cat;
			}
		}


		if ( ! $all_day ) {
			//In case miscrosoft says allday
			$ms_allday = $event->getProperty( 'X-MICROSOFT-CDO-ALLDAYEVENT' );
			if ( ! empty( $ms_allday[1] ) && $ms_allday[1] == 'TRUE' ) {
				$post['EventAllDay'] = 'yes';
			}
		} else {
			$post['EventAllDay'] = 'yes';
		}


		//if we are update or creating an event
		if ( $id = $this->check_for_existing( $post ) ) {
			$post['ID'] = $id;
			//if this event was previously deleted ignore it
		} elseif ( $this->check_for_deleted( $post ) ) {
			return;
		}


		$scrub = new Tribe__Events__Ical_Importer__Submission_Scrubber( $post );
		$post  = $scrub->scrub();

		$this->events[] = $post;
	}

	/**
	 * Get Geo Coordinates
	 *
	 * Check for the GEO property in an event, if not present
	 * attempt to retreive location for Google
	 *
	 * @param array $event
	 *
	 * @return string $lat;$long|bool
	 *
	 */
	public function get_geo_coordinates( $event ) {
		$geo = $event->getProperty( 'GEO' );
		if ( ! empty( $geo ) ) {
			 return $geo;
		}

		$location = tribe_ical_replace_linebreaks_with_spaces( $event->getProperty( 'LOCATION' ) );
		if ( empty( $location ) ) {
			return false;
		}

		//get previously fetched coordinates
		if ( empty( self::$retrieved_geo_coordinates ) ) {
			self::$retrieved_geo_coordinates = (array) get_transient( 'tribe-events-ical-importer-geo' );
		}


		if ( array_key_exists( $location, self::$retrieved_geo_coordinates ) ) {
			return self::$retrieved_geo_coordinates[ $location ];
		}

		//get a new location coordinates
		$geo = wp_remote_get( 'http://maps.googleapis.com/maps/api/geocode/json?address=' . urlencode( $location ) );

		if ( is_wp_error( $geo ) ) {
			self::$retrieved_geo_coordinates[ $location ] = false;

			return false;
		}

		// Ensure $geo contains the expected array (it could be a WP_Error) before tring to parse
		if ( is_array( $geo ) && ! empty( $geo['body'] ) ) {
			$geo = json_decode( $geo['body'] );

			if ( ! empty( $geo->results[0]->geometry->location->lat ) ) {
				$lat  = $geo->results[0]->geometry->location->lat;
				$long = $geo->results[0]->geometry->location->lng;

				return self::$retrieved_geo_coordinates[ $location ] = "$lat;$long";
			}
		} else {
			self::$retrieved_geo_coordinates[ $location ] = false;

			return false;
		}
	}

	/**
	 * Recurrence Meta
	 *
	 * Create the meta data array to save this event as a recurring event
	 *
	 * @param string $rrule - ical formatted rrule
	 * @param array  $event - event data so far
	 *
	 * @example FREQ
	 *    * DAILY
	 *    * WEEKLY
	 *    * MONTHLY
	 *    * YEARLY
	 *    * HOURLY
	 *    * MINUTELY
	 *
	 * @example COUNT - number of events
	 *
	 * @example UNTIL - ical formatted ending date
	 *
	 * @example BYDAY - comma separated
	 *    * SU,MO,TU,WE,TH,FR,SA - WEEKLY
	 *    * 1FR,-1FR - MONTHLY (first and last friday of the month)
	 *
	 * @example BYMONTHDAY
	 *    * 2,-2 - MONTHLY comma separated ( second and second to last day of month )
	 *
	 * @example BYYEARDAY
	 *    * 1,90 - YEARLY comma separated days of the year
	 *
	 * @example BYMONTH
	 *    * 6,7 - YEARLY comma separated months
	 *
	 * @example BYWEEKNO
	 *    * 20 - YEARLY commas separated week of the year
	 *
	 * @example INTERVAL
	 *    * 2 - every other ( Month or week )
	 *
	 * @example BYSETPOS
	 *    * -2 - MONTHLY position of day of the month ( Second to last day )
	 *
	 * @return array
	 *
	 */
	public function recurrenceMeta( $rrule, $event ) {

		$meta_object = new Tribe__Events__Ical_Importer__Recurrence_Meta( $rrule, $event );
		$meta        = $meta_object->get_meta();

		return $meta;
	}

	/**
	 * Check For Existing
	 *
	 * Checks for an existing Event with the same uid
	 * Also checks for one with same time and title
	 *
	 * @param array $event
	 *
	 * @return bool|int - false if none otherwise the id of the event
	 *
	 */
	public function check_for_existing( $event ) {
		global $wpdb;

		$posts = get_posts( array(
			'post_type'   => Tribe__Events__Main::POSTTYPE,
			'post_status' => get_post_stati(),
			'meta_query'  => array(
				array(
					'key'   => '_uid',
					'value' => $event['_uid'],
				),
			),
			'fields'      => 'ids',
			'numberposts' => '1',
		) );


		if ( ! empty( $posts[0] ) ) {

			//use the parent id of the event series
			if ( ! $this->manual() && $this->support_recurrence() ) {
				$parent_id = get_post( $posts[0] )->post_parent;
				if ( ! empty( $parent_id ) ) {
					return $parent_id;
				}
			}

			return $posts[0];
		}

		$possible_duplicates = $wpdb->get_col( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_title = %s AND post_type = %s", $event['post_title'],
			Tribe__Events__Main::POSTTYPE ) );

		$start = date( Tribe__Date_Utils::DBDATETIMEFORMAT, strtotime( $event['EventStartDate'] . ' ' . $event['EventStartHour'] . ':' . $event['EventStartMinute'] . ':00' ) );

		foreach ( $possible_duplicates as $post_id ) {

			if ( get_post_meta( $post_id, '_EventStartDate', true ) == $start ) {

				//use the parent id of the event series
				if ( ! $this->manual() && $this->support_recurrence() ) {
					$parent_id = get_post( $post_id )->post_parent;
					if ( ! empty( $parent_id ) ) {
						return $parent_id;
					}
				}

				return $post_id;
			}
		}

		return false;
	}

	/**
	 * Check For Deleted
	 *
	 * Checks for a deleted Event with the same uid
	 * Also checks for one with same time and title
	 *
	 * @param array $event
	 *
	 * @return bool - true if a deleted exists, false if not
	 *
	 */
	public function check_for_deleted( $event ) {
		global $wpdb;

		$posts = get_posts( array(
			'post_type'   => Tribe__Events__Ical_Importer__Main::DELETED_POST_TYPE,
			'post_status' => get_post_stati(),
			'meta_query'  => array(
				array(
					'key'   => '_uid',
					'value' => $event['_uid'],
				),
			),
			'fields'      => 'ids',
			'numberposts' => '1',
		) );

		if ( ! empty( $posts[0] ) ) {
			return true;
		}

		$possible_duplicates = $wpdb->get_col( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_title = %s AND post_type = %s AND post_status IN ( 'trash' )",
			$event['post_title'], Tribe__Events__Ical_Importer__Main::DELETED_POST_TYPE ) );

		$start = date( Tribe__Date_Utils::DBDATETIMEFORMAT, strtotime( $event['EventStartDate'] . ' ' . $event['EventStartHour'] . ':' . $event['EventStartMinute'] . ':00' ) );

		foreach ( $possible_duplicates as $post_id ) {
			if ( get_post_meta( $post_id, '_EventStartDate', true ) == $start ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * If we are doing a manual (or "one-time") import.
	 *
	 * @return bool
	 */
	public function manual() {
		if ( $this->args['manual'] ) {
			return true;
		}

		return false;
	}

	/**
	 * Indicates if we should support recurrence by forming chains of recurring events when an
	 * RRULE is set.
	 *
	 * Defaults to true when Events Calendar PRO is also active.
	 *
	 * @return bool
	 */
	public function support_recurrence() {
		$pro_exists = class_exists( 'Tribe__Events__Pro__Main' );

		if ( ! $pro_exists ) {
			return false;
		}

		/**
		 * Even if Events Calendar PRO is installed and activated we may prefer to
		 * import events with an RRULE individually rather than as a chain of recurring
		 * events.
		 *
		 * @var bool $support_recurrence
		 */
		return (bool) apply_filters( 'tribe_events_ical_importer_support_recurrence', $pro_exists );
	}

	/**
	 * There are a number of existing challenges in terms of converting events with RRULEs into
	 * full-blown recurring events as understood by Events Calendar PRO that still need to be
	 * resolved; this method turns off recurrence support so that any recurring events in the iCal
	 * source are pulled in as individual events.
	 */
	protected function deactivate_full_recurrence_support() {
		add_filter( 'tribe_events_ical_importer_support_recurrence', '__return_false' );
	}

	/**
	 * Get Raw Events By Range
	 *
	 * Retrieves the vcalendar event arrays
	 * Uses the start and end args to retrieve events only in specified time frame
	 *
	 * @return array
	 */
	public function get_raw_events_by_range() {

		$date        = strtotime( $this->args['start'] );
		$start_month = date( 'm', $date );
		$start_year  = date( 'Y', $date );
		$start_day   = date( 'd', $date );

		$date      = strtotime( $this->args['end'] );
		$end_month = date( 'm', $date );
		$end_year  = date( 'Y', $date );
		$end_day   = date( 'd', $date );

		return (array) $this->selectComponents( $start_year, $start_month, $start_day, $end_year, $end_month, $end_day, 'vevent' );
	}

	/**
	 * Generates UTC start/end dates from the start/end date based on timezone
	 *
	 * @param array  $event    Event data
	 * @param string $timezone Timezone of the start/end date used to generate UTC start/end dates
	 *
	 * return array
	 */
	public function generate_utc_from_timezone_date( $event, $timezone ) {
		foreach ( array( 'Start', 'End' ) as $which ) {
			$date   = $event[ 'Event' . $which . 'Date' ];
			$hour   = $event[ 'Event' . $which . 'Hour' ];
			$minute = $event[ 'Event' . $which . 'Minute' ];
			$date   = "{$date} {$hour}:{$minute}:00";

			$event[ 'Event' . $which . 'DateUTC' ] = Tribe__Events__Timezones::to_utc( $date, $timezone );
		}

		return $event;
	}

	/**
	 * Generates start/end dates from the UTC start/end date for a timezone
	 *
	 * @param array  $event    Event data
	 * @param string $timezone Timezone of the desired start/end date generated from the UTC start/end dates
	 *
	 * return array
	 */
	public function generate_timezone_date_from_utc( $event, $timezone ) {
		foreach ( array( 'Start', 'End' ) as $which ) {
			$date = $event[ 'Event' . $which . 'DateUTC' ];
			$date = Tribe__Events__Timezones::to_tz( $date, $timezone );

			if ( empty( $event['EventAllDay'] ) || 'no' === $event['EventAllDay'] ) {
				$event[ 'Event' . $which . 'Date' ] = date( Tribe__Date_Utils::DBDATEFORMAT, strtotime( $date ) );
			}

			$event[ 'Event' . $which . 'Hour' ]   = date( 'H', strtotime( $date ) );
			$event[ 'Event' . $which . 'Minute' ] = date( 'i', strtotime( $date ) );
		}

		return $event;
	}

	/**
	 * Sets the start/end dates as the UTC start/end dates
	 *
	 * @param array $event Event data
	 *
	 * return array
	 */
	public function store_date_as_utc( $event ) {
		foreach ( array( 'Start', 'End' ) as $which ) {
			$date                                  = $event[ 'Event' . $which . 'Date' ];
			$hour                                  = $event[ 'Event' . $which . 'Hour' ];
			$minute                                = $event[ 'Event' . $which . 'Minute' ];
			$date                                  = "{$date} {$hour}:{$minute}:00";
			$event[ 'Event' . $which . 'DateUTC' ] = $date;
		}

		return $event;
	}

	/**
	 * Convert a system timezone datetime string to the desired event datetime string
	 *
	 * A conversion only occurs if the system timezone is NOT UTC
	 *
	 * @param string $date_string     Date string to possibly convert
	 * @param string $system_timezone System timezone
	 * @param string $event_timezoen  Timezone of event in ical feed
	 *
	 * @return string
	 */
	public function maybe_convert_from_system_timezone( $date_string, $system_timezone, $event_timezone ) {
		// if the current system timezone is NOT UTC, we need to correct the the adjustment by iCalCreator
		// by converting the x_start_str from the system timezone to UTC and then converting THAT UTC
		// string to the timezone of the event in the feed
		if ( 'UTC' === $system_timezone ) {
			return $date_string;
		}

		$utc_start_str = Tribe__Events__Timezones::to_utc( date( Tribe__Date_Utils::DBDATETIMEFORMAT, strtotime( $date_string ) ), $system_timezone );

		$date_string = date( 'Ymd\THis', strtotime( Tribe__Events__Timezones::to_tz( $utc_start_str, $event_timezone ) ) );

		return $date_string;
	}

	/**
	 * @param iCalVEVENT $event
	 *
	 * @return array
	 */
	private function get_venue_data_array( iCalVEVENT $event ) {
		$google_maps_enabled = Tribe__Events__Importer__Options::getOption( 'ical_import_enable_GoogleMaps', false ) ? 'true' : 'false';
		$data_array = array(
			'Venue'            => tribe_ical_replace_linebreaks_with_spaces( $event->getProperty( 'LOCATION' ) ),
			'ShowMap'          => $google_maps_enabled,
			'ShowMapLink'      => $google_maps_enabled,
			'EventShowMap'     => $google_maps_enabled,
			'EventShowMapLink' => $google_maps_enabled,
		);

		return $data_array;
	}

	/**
	 * Returns the parsed calendar timezone.
	 *
	 * This timezone might not be the same found in the orginal iCal file.
	 * Some iCal files will specify timezones that are not PHP recognized: these timezones will be converted to a timezone PHP can parse.
	 *
	 * @return string
	 */
	public function get_calendar_timezone() {
		return $this->timezone;
	}
}//end class
