<?php
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

if ( class_exists( 'Tribe__Events__Ical_Importer__Filters' ) ){
	return;
}

/**
 * Tribe Events iCal Importer Filters
 *
 * Allow filtering by events source in posts lists and APM
 * Also displays a message to let you know you editing a imported event
 *
 *
 * @package Tribe__Events__Ical_Importer__Main
 *
 */
class Tribe__Events__Ical_Importer__Filters {

	/**
	 * Apm Filters
	 *
	 * Filter to add to the apm object
	 *
	 * @var array
	 */
	public $apm_filters = array();

	/**
	 * APM Filter Key
	 *
	 * Key for the apm filter field
	 *
	 * @var string
	 */
	const APM_FILTER_KEY = 'ical_source';

	/**
	 * Imported Meta Key
	 *
	 * Key used to save the event source and know it was an imported event
	 *
	 * @var string
	 */
	const IMPORTED_META_KEY = '_EventIcalSource';


	 /**
	 * Saved Import Meta Key
	 *
	 * Key used to save the Saved Import origin for later filtering
	 *
	 * @var string
	 */
	const SAVED_IMPORT_META_KEY = '_EventSavedImport';


	/**
	 * iCal Sources
	 *
	 * Hold the retrieved iCal Sources
	 *
	 * @var array
	 */
	public static $ical_sources;


	/**
	 * Constructor
	 *
	 * @uses Runs on self::init();
	 *
	 */
	public function __construct() {

		$this->addActions();
		$this->addFilters();
	}


	/**
	 * Add Actions
	 *
	 * Add actions where neccessary
	 *
	 * @return void
	 */
	public function addActions() {

		//apm
		add_action( 'tribe_cpt_filters_init', array( $this, 'filtersAddToAPM' ) );

		//post list columns
		if ( ! class_exists( 'Tribe__Events__Pro__Main' ) ) {
			add_action( 'restrict_manage_posts', array( $this, 'postListFilters' ) );
		}

		//post list items
		add_action( 'pre_get_posts', array( $this, 'postListQuerySource' ), 999 );

		//admin messages
		add_action( 'admin_notices', array( $this, 'adminMessage' ) );


	}


	/**
	 * Add Filters
	 *
	 * Add filters where neccessary
	 *
	 * @return void
	 */
	public function addFilters() {
		add_filter( 'tribe_query_options', array( $this, 'APMQueryOptionsFilter' ), 9, 3 );
	}


	/**
	 * Admin Message
	 *
	 * Display a warning letting the user know the event came from an import
	 *
	 * @return void
	 *
	 */
	public function adminMessage() {
		global $pagenow;

		if ( $pagenow != 'post.php'
			|| get_post_type() != Tribe__Events__Main::POSTTYPE ) {
				 return;
		}

	 	if ( ! get_post_meta( get_the_ID(), self::IMPORTED_META_KEY, true ) ) {
	 		return;
	 	}

		?>
		<div class="updated">
			<p>
				<?php esc_html_e( 'This event came from an import. If it gets imported again, your changes will be overwritten', 'tribe-events-ical-importer' ); ?>
			</p>
		</div>
		<?php
	}


	/**
	 * Post List Query Source
	 *
	 * Adds a meta query to the post lists query for ical source is one is selected to
	 * filter with
	 *
	 * @param WP_Query $query
	 *
	 * @return void
	 */
	public function postListQuerySource( $query ) {
		global $pagenow;

		if ( $pagenow != 'edit.php'
		    || empty( $query->query_vars['post_type'] )
		    || $query->query_vars['post_type'] != Tribe__Events__Main::POSTTYPE
			|| ( empty( $_REQUEST['ical_source'] ) && empty( $_REQUEST['saved_import'] ) ) ) {
				 return;
		}

		if ( ! empty( $_REQUEST['ical_source'] ) ) {
			$meta[] = array(
				'key' => self::IMPORTED_META_KEY,
				'value' => trim( urldecode( $_REQUEST[ 'ical_source' ] ) ),
			);
		}

		if ( ! empty( $_REQUEST['saved_import'] ) ) {
			$meta[] = array(
					'key' => self::SAVED_IMPORT_META_KEY,
					'value' => trim( urldecode( $_REQUEST['saved_import'] ) ),
			);
		}

		$query->set( 'meta_query', $meta );

	}


	/**
	 * Post List Filters
	 *
	 * Dropdown to select an imported events source or just imported ones
	 *
	 * @return void
	 *
	 */
	public function postListFilters() {

		if ( get_post_type() != Tribe__Events__Main::POSTTYPE ) {
			return;
		}

		?>
		<select name="ical_source">
			<option value>
				<?php esc_html_e( 'View All Sources', 'tribe-events-ical-importer' ); ?>
			</option>
			<?php
			foreach ( self::icalSourcesGet() as $source ) {
				printf( '<option value="%s">%s</option>', esc_attr( urlencode( $source ) ), $source );
			}
			?>
		</select>
		<?php

	}


	/**
	 * iCal Sources Get
	 *
	 * Retrieve all unique iCal sources from currently imported Events
	 *
	 * @return array ('meta_value' => 'meta_value );
	 */
	public static function icalSourcesGet() {
		global $wpdb;
		if ( ! empty( self::$ical_sources ) ) {
			return self::$ical_sources;
		}

		$q = $wpdb->prepare( "SELECT DISTINCT $wpdb->postmeta.meta_value AS source FROM $wpdb->postmeta INNER JOIN $wpdb->posts ON ($wpdb->posts.ID = $wpdb->postmeta.post_id) WHERE post_type= %s AND meta_key = %s", Tribe__Events__Main::POSTTYPE, self::IMPORTED_META_KEY );

		$retrieved = $wpdb->get_col( $q );

		$sources = array();
		foreach ( $retrieved as $source ) {
			$sources[ $source ] = $source;
		}

		return self::$ical_sources = $sources;
	}

	/**
	 * APM Query Options Filter
	 *
	 * Remove all non applicable options form the APM dropdown
	 *
	 * @param array $options
	 * @param string $key
	 * @param array $filter
	 *
	 * @return array
	 */
	public function APMQueryOptionsFilter( $options, $key, $filter ) {
		if ( $key != self::APM_FILTER_KEY ) {
			return $options;
		}

		$options = array(
			'is' => __( 'Is', 'tribe-apm' ),
			'not' => __( 'Is Not', 'tribe-apm' ),
		);
		return $options;
	}


	/**
	 * Filters add to APM
	 *
	 * Add a source filters to the Tribe_APM object
	 *
	 * @param Tribe_APM $apm
	 *
	 * @return void
	 */
	public function filtersAddToAPM( $apm ) {
		$this->apm_filters = array(
			self::APM_FILTER_KEY => array(
            	'name'        => __( 'iCal Source', 'tribe-events-ical-importer' ),
            	'meta'        => self::IMPORTED_META_KEY,
            	'sortable'    => true,
            	'options'     => self::icalSourcesGet(),
			),
		);

		$apm->add_filters( $this->apm_filters );

	}


	/********** SINGLETON FUNCTIONS **********/

	/**
	 * Instance of this class for use as singleton
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::$instance = self::instance();
	}

	/**
	 * Get (and instantiate, if necessary) the instance of the class
	 *
	 * @static
	 * @return Tribe__Events__Ical_Importer__Filters
	 */
	 public static function instance() {
		if ( ! self::$instance instanceof self ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

}