<?php

// Don't load directly
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

if ( class_exists( 'Tribe__Events__Importer__Importer' ) ) {
	return;
}

/**
 * Tribe Events Importer
 *
 * Class extended to create individual Importers
 *
 * @package Tribe__Events__Importer__Importer
 *
 * @uses Must be extended by another class
 *
 */
abstract class Tribe__Events__Importer__Importer extends Tribe__Events__Importer__Gui{

	/**
	 * Importer Register
	 *
	 * Holdes the register class in a var so we can use it
	 *
	 * @var Tribe__Events__Importer__Registrar
	 */
	public $importer_register;

	/**
	 * Count Inserted Events
	 *
	 * Used to keep track of how many events we are importing
	 *
	 * @var int
	 */
	private static $count_inserted_events = 0;

	/**
	 * Count Updated Events
	 *
	 * Used to keep track of how many events we are updating
	 *
	 * @var int
	 */
	private static $count_updated_events = 0;

	/**
	 * Count scanned events
	 *
	 * Used to keep track of how many events were scanned (not all events will
	 * necessarily be imported or updated).
	 *
	 * @var int
	 */
	private static $count_scanned_events = 0;

	/**
	 * Saving Events
	 *
	 * If we are currently running an import
	 *
	 * @var bool
	 */
	protected $saving_events = false;

	/**
	 * The child plugin name.
	 * @var string $pluginName
	 */
	protected $pluginName;


	/**
	 * The slug and main plugin file
	 * @var string
	 */
	 protected $pluginDirFile;

	/**
	 * The child plugin short name.
	 * @var string $pluginShortName
	 */
	protected $pluginShortName;

	/**
	 * The required TEC version.
	 * @var string $requiredTecVersion
	 */
	protected $requiredTecVersion;

	/**
	 * The plugin's current version.
	 * @var string $currentVersion
	 */
	protected $currentVersion;

	/**
	 * The path to the plugin's main directory.
	 * @var string $pluginPath
	 */
	protected $pluginPath;

	/**
	 * The URL of the plugin's main directory.
	 * @var string $pluginUrl
	 */
	protected $pluginUrl;

	/**
	 * The plugin slug, used in various places.
	 * @var string $pluginSlug
	 */
	protected $pluginSlug;

	/**
	 * Any errors that may have come up during a function.
	 * @var array $errors
	 */
	protected $errors;

	/**
	 * Any alert messages that are not errors that may have come up during a function.
	 * @var array $messages
	 */
	protected $messages;

	/**
	 * The array that will be built into JSON to send to the user in response to AJAX requests.
	 * @var array $response
	 */
	protected $response;

	/**
	 * A basic array of possible events to import.
	 * @var $possibleEvents
	 */
	protected $possibleEvents;



	/**
	 * Must be defined as a singleton method by the child class.
	 *
	 * @self
	 * @return Tribe__Events__Importer__Importer
	 */
	public static function instance(){}

	/**
	 * The method used by the child class to add WordPress actions.
	 *
	 *
	 * @return void
	 */
	abstract protected function addActions();

	/**
	 * The method used by the child class to add WordPress filters.
	 *
	 *
	 * @return void
	 */
	abstract protected function addFilters();

	/**
	 * Abstract method that processes the import form.
	 *
	 */
	abstract public function processImportSubmission( $args = null );

	/**
	 * Abstract method that is used to get event data from a source.
	 * The source could be an HTTP request, an XML document, or any other number of types of sources.
	 * It should return data that can be parsed by the setEventData() method.
	 *
	 */
	abstract protected function getEventsData();


	/**
	 * Create the import submenu page.
	 *
	 *
	 * @return void
	 */
	abstract public function doImportPage();


	/**
	 * The class constructor function.
	 *
	 *
	 * @return void
	 */
	protected function __construct() {
		$this->className = get_class( $this );
		$this->importer_register = Tribe__Events__Importer__Registrar::instance();

		$this->_addActions();
		$this->_addFilters();

		$this->addActions();
		$this->addFilters();
	}

	/**
	 * The method used to add the actions necessary for the class to work.
	 *
	 *
	 * @return void
	 */
	protected function _addActions() {

		$this->_ajax_actions();

		$this->_gui_actions();

		add_action( 'tribe_events_' . $this->pluginSlug . '_do_scheduled_import', array( $this, 'processImportSubmission' ), 10, 1 );

		add_action( 'admin_enqueue_scripts', array( $this, '_enqueueScriptsAndStyles' ) );

		//because some are created during recurrence meta saving
		add_action( 'save_post_' . Tribe__Events__Main::POSTTYPE, array( $this, 'countInsertedEvents' ), 9, 3 );

	}

	/**
	 * The method used to add the filters necessary for the class to work.
	 * This can be overridden by a child class, but it is highly recommended that it incorporate
	 * parent::addFilters() so that these essential ones are included. :-p
	 *
	 *
	 * @return void
	 */
	protected function _addFilters() {
		add_filter( 'tribe-events-importexport-import-apis', array( $this, 'addEventImporter' ) );
	}

	/**
	 * Enqueue the scripts and styles.
	 *
	 *
	 * @return void
	 */
	public function _enqueueScriptsAndStyles() {
		if ( Tribe__Events__Importer__Plugin::$admin->get_active_tab() == $this->pluginSlug ) {
			Tribe__Events__Template_Factory::asset_package( 'tribe-events-import' );
		}
	}

	/**
	 * The function used to compare versions and initialize the addon.
	 *
	 * @param array $plugins
	 * @return array
	 */
	public function initAddon( $plugins ) {
		$plugins[ $this->className ] = array(
			'plugin_name'      => $this->pluginName,
			'required_version' => $this->requiredTecVersion,
			'current_version'  => $this->currentVersion,
			'plugin_dir_file'  => $this->pluginDirFile,
		);
		return $plugins;
	}

	/**
	 * The method is used to add the event origin slug to the event's audit trail.
	 *
	 * @return string The origin slug for a given importer.
	 */
	public function addEventOrigin() {
		return $this->pluginSlug;
	}

	/**
	 * Add Event Importer
	 *
	 * @param array $import_apis
	 * @return array An array representing this specific event importer.
	 */
	public function addEventImporter( $import_apis ) {
		$import_apis[] = array(
			'slug' => $this->pluginSlug,
			'name' => $this->pluginShortName,
		);
		return $import_apis;
	}


	/**
	 * Count Inserted Events
	 *
	 * Runs during the wp_insert_post to count all new events added
	 * Done this way because some events may be created during saving of recurrence meta
	 *
	 * @param int $post_id
	 * @param WP_Post $post
	 * @param bool $update - if we are updating a post ignore
	 *
	 * @return void
	 */
	public function countInsertedEvents( $post_id, $post, $update ) {

		if ( ! $this->saving_events ) {
			return;
		}

		if ( Tribe__Events__Main::POSTTYPE != $post->post_type ) {
			return;
		}

		if ( $update ) {
			self::$count_updated_events++;
		} else {
			self::$count_inserted_events++;
		}
	}

	/**
	 * Reset Counts
	 *
	 * Rests the inserted and updated counts
	 *
	 * @uses mostly for unit tests to have a fresh start each time
	 *
	 * @return void
	 */
	public static function resetCounts() {
		self::$count_inserted_events = 0;
		self::$count_updated_events = 0;
		self::$count_updated_events = 0;
	}


	/**
	 * Save Events
	 *
	 * Turns the previously formatted events array into saved events
	 *
	 * @param array $events - previously formatted array ready for tribe_create_event()
	 * @param array $args
	 *
	 * @return array( 'inserted' => int, 'updated' => int, 'scanned' => int ) The number of events that were imported.
	 */
	protected function saveEvents( $events, $args = array() ) {
        $this->saving_events = true;

		if ( empty( $args['post_status'] ) ) {
			$args['post_status'] = Tribe__Events__Importer__Options::get_default_post_status( 'ical' );
		}

		//cache
		$possible_parents = array();
		$found_organizers = array();
		$found_venues     = array();

		$overwrite = ! empty( $args[ 'recurring_overwrite' ] ) && 'checked' === $args[ 'recurring_overwrite' ] ? true : false;

		//if we have no non recurring events the message may be different
		$non_recurring = false;

		foreach ( $events as $event ) {
			// Track how many events we scanned with a view to possibly importing/updating the local copy
			self::$count_scanned_events++;

			/**
			 * Should events that have previously been imported be overwritten?
			 *
			 * By default this is turned off (since it would reset the post status, description
			 * and any other fields that have subsequently been edited) but it can be enabled
			 * by returning true on this filter.
			 *
			 * @var bool $overwrite
			 * @var int  $event_id
			 */
			if ( ! empty( $event['ID'] ) && ! apply_filters( 'tribe_events_ical_importer_overwrite_existing_events', $overwrite, $event['ID'] ) ) {
				continue;
			}

			if ( empty( $event[ 'recurrence' ] ) ) {
				$non_recurring = true;
			}

			if ( empty( $event['post_status'] ) ) {
				$event['post_status'] = $args['post_status'];
			}

			//set the parent
			if ( ! class_exists( 'Tribe__Events__Pro__Main' ) ) {
				if ( ! empty( $event[ 'ID' ] ) && ( $id = wp_get_post_parent_id( $event[ 'ID' ] ) ) ) {
					$event['post_parent'] = $id;
				} elseif ( ! empty( $event['parent_uid'] ) && ( $k = array_search( $event['parent_uid'], $possible_parents ) ) ) {
					$event['post_parent'] = $k;
				}

			//PRO version will already be set to parent of an existing series during check for duplicate
			} elseif ( ! empty( $event['parent_uid'] ) ) {
				if ( $k = array_search( $event['parent_uid'], $possible_parents ) ) {
					$event['post_parent'] = $k;
				}
			}

			//if we should create a venue or use existing
			if ( ! empty( $event['Venue']['Venue'] ) ) {
				$v_id = array_search( $event['Venue']['Venue'], $found_venues );
				if ( $v_id !== false ) {
					$event['EventVenueID'] = $v_id;
				} elseif ( $venue = get_page_by_title( $event['Venue']['Venue'], 'OBJECT', Tribe__Events__Main::VENUE_POST_TYPE ) ) {
					$found_venues[ $venue->ID ] = $event['Venue']['Venue'];
					$event['EventVenueID']      = $venue->ID;
				} else {
					$event['EventVenueID'] = Tribe__Events__Venue::instance()->create( $event['Venue'], $args['post_status'] );
				}
				unset( $event['Venue'] );
			}

			//if we should create an organizer or use existing
			if ( ! empty( $event['Organizer']['Organizer'] ) ) {
				$o_id = array_search( $event['Organizer']['Organizer'], $found_organizers );
				if ( $o_id !== false ) {
					$event['EventOrganizerID'] = $o_id;
				} elseif ( $organizer = get_page_by_title( $event['Organizer']['Organizer'], 'OBJECT', Tribe__Events__Main::ORGANIZER_POST_TYPE ) ) {
					$found_organizers[ $organizer->ID ] = $event['Organizer']['Organizer'];
					$event['EventOrganizerID']          = $organizer->ID;
				} else {
					$event['EventOrganizerID'] = Tribe__Events__Organizer::instance()->create( $event['Organizer'], $args['post_status'] );
				}
				unset( $event['Organizer'] );
			}

			$event['post_type'] = Tribe__Events__Main::POSTTYPE;

			if ( ! empty( $event['ID'] ) ) {
				$event['ID'] = tribe_update_event( $event['ID'], $event );
			} else {
				$event['ID'] = tribe_create_event( $event );
			}

			//add post parent possibility
			if ( empty( $event['parent_uid'] ) ) {
				$possible_parents[ $event['ID'] ] = $event['_uid'];
			}

			update_post_meta( $event['ID'], '_uid', $event['_uid'] );

			//Save the meta data in case of updating to pro later on
			if ( ! class_exists( 'Tribe__Events__Pro__Main' ) ) {
				if ( ! empty( $event['recurrence'] ) ) {
					update_post_meta( $event['ID'], '_EventRecurrenceRRULE', $event['recurrence'] );
				}
			}

			$terms = array();
			if ( ! empty( $event['categories'] ) ) {
				foreach ( $event['categories'] as $cat ) {
					if ( ! $term = term_exists( $cat, Tribe__Events__Main::TAXONOMY ) ) {
						$term = wp_insert_term( $cat, Tribe__Events__Main::TAXONOMY );
						$terms[] = (int) $term['term_id'];
					} else {
						$terms[] = (int) $term['term_id'];
					}
				}
			}

			//if we are setting all events to a category specified in saved import
			if ( ! empty( $args['import_category'] ) ) {
				$terms[] = (int) $args['import_category'];
			}

			wp_set_object_terms( $event['ID'], $terms, Tribe__Events__Main::TAXONOMY, false );
		}

		$counts = array(
			'inserted' => self::$count_inserted_events,
			'updated'  => self::$count_updated_events,
			'scanned'  => self::$count_scanned_events,
		);

		if ( ! $non_recurring ) {
			$counts[ 'recurring' ] = 1;
		}

		$this->saving_events = false;

		return $counts;
	}
}
