<?php
// Don't load directly
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

if ( class_exists( 'Tribe__Events__Ical_Importer__Main' ) ) {
	return;
}

/**
 * Tribe Events iCal Importer
 *
 * Main Class for the Tribe Events iCal Importer
 *
 * @package Tribe__Events__Ical_Importer__Main
 *
 */
class Tribe__Events__Ical_Importer__Main extends Tribe__Events__Importer__Importer {

	/**
 	* Deleted Post Type
 	*
 	* The post type deleted events will become upon deletion
 	*
 	* @var string
 	*/
	const DELETED_POST_TYPE = 'deleted_event';

	/**
 	* Instance of this class for use as singleton
 	*/
	protected static $instance;

	/**
	 * plugin_path
	 *
	 * @static
	 * @var string
	 */
	public static $plugin_path;

	/**
	 * iCal Source
	 *
	 * The calname from the iCal file
	 *
	 * @var string
	 */
	 public static $ical_source;

	/**
	 * Saved Import Key
	 *
	 * The key of the running saved import
	 * Used to allow filtering by this key later
	 *
	 * @var float
	 */
	 public $saved_import_key = null;

	/**
	 * Memory Limit
	 *
	 * Adjustable memory limit for the script to run within
	 *
	 * @var string
	 */
	public $memory_limit = '256M';

	/**
	 * Constructor
	 *
	 *
	 */
	protected function __construct() {
		$this->pluginName = __( 'The Events Calendar: iCal Importer', 'tribe-events-ical-importer' );
		$this->pluginShortName = 'iCal';
		$this->requiredTecVersion = '4.2beta1';
		$this->currentVersion = '4.2.3';
		$this->pluginPath = trailingslashit( dirname( dirname( dirname( __FILE__ ) ) ) );
		$this->pluginUrl = trailingslashit( plugins_url() ) . basename( $this->pluginPath );
		$this->importPageName = 'Import from iCal';
		$this->pluginSlug = 'ical-importer';
		$this->pluginDirFile = 'events-ical-importer/the-events-calendar-ical-importer.php';
		$this->possibleEvents = array();
		$this->location = null;
		$this->radius = null;
		$this->category = null;
		$this->startDate = null;
		$this->endDate = null;
		$this->keywords = null;
		$this->nougc = null;
		$this->page = 1;
		$this->icalCategories = array();

		self::$plugin_path = $this->pluginPath;

		parent::__construct();

	}

	/**
	 * Determines whether or not this plugin should be bootstrapped
	 */
	public static function should_bootstrap() {
		return ( is_admin() || ( defined( 'DOING_CRON' ) && DOING_CRON ) || defined( 'WP_TESTS_DOMAIN' ) || ( defined( 'WP_CLI' ) && WP_CLI ) );
	}

	/**
	 * Add actions.
	 *
	 *
	 * @return void
	 */
	protected function addActions() {
		//We currently don't want out of date messages to display
		//add_filter( 'tribe_tec_addons', array( $this, 'initAddon' ) );

		if ( self::should_bootstrap() ) {
			add_action( 'init', array( $this, 'register_importer' ) );

			add_action( 'admin_enqueue_scripts', array( $this, 'jsCss' ) );

			add_action( 'before_delete_post', array( $this, 'deletedEventSave' ) );

			add_action( 'init', array( $this, 'registerDeletedPostType' ) );

			//because some are created during recurrence meta saving
			add_action( 'save_post_' . Tribe__Events__Main::POSTTYPE, array( $this, 'metaDataSet' ), 9, 3 );

			$recurring_imports_notice = Tribe__Events__Ical_Importer__Admin__Notice__Failed_Recurring_Imports::instance();
			add_action( 'admin_notices', array( $recurring_imports_notice, 'maybe_display' ) );
			add_action( 'admin_init', array( $recurring_imports_notice, 'maybe_dismiss' ), 5 );
			
			add_action( 'admin_init', array( $this, 'dismissSavedImportsWarning' ) );

			add_action( 'tribe_events_importexport_content_tab_ical-importer', array( $this, 'tab_content' ), 1 );

		}

		add_action( 'wp_before_admin_bar_render', array( $this, 'addToolbarItem' ), 11 );
	}

	/**
	 * Register Importer
	 *
	 * Adds the importer to the CSV Import tabbed structure
	 *
	 *
	 * @return void
	 */
	public function register_importer() {
		add_filter( 'tribe_import_tabs', array( $this, 'add_ical_tab' ) );
		add_action( 'tribe_import_render_tab_' . $this->pluginSlug, array( $this, 'tab_content' ) );

		$settings = Tribe__Events__Ical_Importer__General_Settings::instance();
		add_filter( 'tribe_import_available_options', array( $settings, 'add_import_available_fields' ), 15 );
		add_action( 'tribe_import_general_settings', array( $settings, 'add_settings_fields' ), 25 );

	}

	public function add_ical_tab( $tabs ) {
		$tabs[ __( 'iCal', 'tribe-events-ical-importer' ) ] = $this->pluginSlug;

		return $tabs;
	}
	
	/**
	 * tab_content
	 *
	 * Outputs the content for the Import : Ical tab
	 *
	 * @uses tribe_ical_importer_get_template()
	 *
	 * @return void
	 */
	public function tab_content() {
		tribe_ical_importer_get_template( 'ical-tab-content' );
	}

	/**
	 * Add filters.
	 *
	 * @return void
	 */
	protected function addFilters() {
		add_filter( 'upload_mimes', array( $this, 'allowIcalFiles' ), 99999 );
	}

	/**
	 * Allow Ical Files
	 *
	 * Adds .ics to allow files types
	 *
	 * @param array $mimes
	 *
	 * @return array
	 */
	public function allowIcalFiles( $mimes ) {

		if ( ! array_key_exists( 'ics', $mimes ) ) {
			$mimes['ics'] = 'text/calendar';
		}

		return $mimes;
	}

	/**
	 * Add Tool Bar Item
	 *
	 * Add the iCal link to the toolbar
	 *
	 * @return void
	 *
	 */
	public function addToolbarItem() {
		global $wp_admin_bar;

		$wp_admin_bar->add_menu( array(
			'id' => 'tribe-ical-import',
			'title' => __( 'iCal', 'tribe-events-ical-importer' ),
			'href' => self::get_url(),
			'parent' => 'tribe-events-import',
		) );

	}

	/**
	 * Set Limits
	 *
	 * Set the time an memory limits to prevent failing out
	 *
	 * @uses called at the beginning of $this->processImportSubmission()
	 *
	 * @return void
	 */
	private function setLimits() {
		set_time_limit( 0 );
		$this->memory_limit = ini_get( 'memory_limit' );
		if ( $this->memory_limit < WP_MAX_MEMORY_LIMIT ) {
			$this->memory_limit = WP_MAX_MEMORY_LIMIT;
			@ini_set( 'memory_limit', $this->memory_limit );
		}

	}


    /**
	 * Meta Data Set
	 *
	 * Runs during the wp_insert_post to set any special meta data
	 * Done this way because some events may be created during saving of recurrence meta
	 *
	 * @param int $post_id
	 * @param WP_Post $post
	 * @param bool $update - if we are updating a post ignore
	 *
	 * @return void
	 */
	public function metaDataSet( $post_id, $post, $update ) {
		if ( ! $this->saving_events ) {
			return;
		}

		update_post_meta( $post_id, Tribe__Events__Ical_Importer__Filters::IMPORTED_META_KEY, self::$ical_source );

		if ( $this->saved_import_key != null ) {
			update_post_meta( $post_id, Tribe__Events__Ical_Importer__Filters::SAVED_IMPORT_META_KEY, $this->saved_import_key );
		}
	}

	/**
	 * Register Deleted Post Type
	 *
	 * Create a post type we can assign copies of deleted events to
	 * Used to prevent reimporting a previously deleted event
	 *
	 * @return void
	 *
	 */
	public function registerDeletedPostType() {
		register_post_type( self::DELETED_POST_TYPE, array(
			'public' => false,
		) );
	}

	/**
	 * Deleted Event Save
	 *
	 * Save a copy of a previously imported event when deleted
	 * Used to prevent reimporting an previously deleted imported event
	 *
	 * @param int $post_id - id of post about to be deleted
	 *
	 * @return void
	 */
	public function deletedEventSave( $post_id ) {

		$post = get_post( $post_id, 'ARRAY_A' );
		if ( $post[ 'post_type' ] != Tribe__Events__Main::POSTTYPE ) {
			return;
		}

		$uid = get_post_meta( $post_id, '_uid', true );
		if ( empty( $uid ) ) {
			return;
		}

		$post[ 'post_type' ] = self::DELETED_POST_TYPE;
		unset( $post[ 'ID' ] );

		$id = wp_insert_post( $post );

		update_post_meta( $id, '_EventStartDate', get_post_meta( $post_id, '_EventStartDate', true ) );
		update_post_meta( $id, '_uid', $uid );

	}

	/**
	 * Js Css
	 *
	 * Cue the css and js files in the appropriate locations
	 *
	 * @return void
	 *
	 */
	public function jsCss() {
		$importer_registrar = Tribe__Events__Importer__Registrar::instance();

		if ( Tribe__Events__Importer__Plugin::$admin->get_active_tab() == $this->pluginSlug ) {
			Tribe__Events__Template_Factory::asset_package( 'datepicker' );
			Tribe__Events__Template_Factory::asset_package( 'admin-ui' );
			Tribe__Events__Template_Factory::asset_package( 'jquery-placeholder' );

			wp_enqueue_media();

			wp_enqueue_style( 'tribe-ical-import-form', $this->pluginUrl . '/src/resources/css/import-form.css', false, false, 'screen' );

			$deps = array( 'jquery', 'jquery-ui-datepicker', 'jquery-ui-progressbar', 'tribe-events-admin' );
			wp_enqueue_script( 'tribe-ical-import-form', $this->pluginUrl . '/src/resources/js/import-form.js', $deps, $this->currentVersion, true );

			wp_localize_script( 'tribe-ical-import-form', 'icalImportNonce', wp_create_nonce( 'ical-import-nonce' ) );
			wp_localize_script( 'tribe-ical-import-form', 'tribeIcalImportPluginSlug', $this->pluginSlug );

			wp_localize_script( 'tribe-ical-import-form', 'tribeIcalImportUrl', $this->get_url() );

			wp_localize_script( 'tribe-ical-import-form', 'tribeIcalImportConfig', $this->js_config() );

		}
	}

	private function js_config() {
		return array(
			'local' => array(
				'edit_recurring' => __( 'Edit Recurring Import', 'tribe-events-ical-importer' ),
			),
		);
	}

	public function get_url() {
		return get_admin_url( false, '/edit.php?post_type=' . Tribe__Events__Main::POSTTYPE.'&page=events-importer&tab=' . $this->pluginSlug );
	}

	/**
	 * Cron Schedule Get
	 *
	 * Get our own filter version of the crons
	 *
	 * @return array
	 */
	public static function cronScheduleGet() {
		$cron_schedules = wp_get_schedules();
		unset( $cron_schedules['twicedaily'] );
		uasort( $cron_schedules, array( __CLASS__, 'sortOnInterval' ) );

		foreach ( $cron_schedules as &$schedule ) {
			$schedule['display'] = str_replace( 'Once ', '', $schedule['display'] );
		}

		return $cron_schedules;
	}

    /**
	 * Sort on Interval
	 *
	 * Sort an array via uasort using the interval field
	 *
	 * @return int
	 */
	public static function sortOnInterval( $a, $b ) {
		return $a['interval'] - $b['interval'];
	}

	/**
	 * Build Possible Event Items
	 *
	 * Turn the events list into items which may be used by the
	 * Tribe__Events__Ical_Importer__Possible_Events_List_Table class to
	 * generate the events table
	 *
	 * @param array $possible_events
	 *
	 * @return string
	 *
	 */
	public function buildPossibleEventsListItems( $possible_events ) {
		$table = new Tribe__Events__Ical_Importer__Possible_Events_List_Table();

		$page = $_REQUEST['paged'];

		$date_format = get_option( 'date_format' );

		foreach ( $possible_events as $count => $event ) {
			if ( $page == 1 ) {
				$count ++; //possible events start at 0
			}

			if ( $count < ( ( $page - 1 ) * 30 ) ) {
				continue;
			}

			if ( $count > ( $page * 30 ) ) {
				break;
			}

			/**
			 * The Ical parsing library would output 0000-00--1 when a data (end or start) is not set and default to Unix beginning of time.
			 */
			$beginning_of_time = '0000-00--1';
			$is_all_day_event  = isset( $event['EventAllDay'] ) && $event['EventAllDay'] === 'yes';
			if ( $is_all_day_event && isset( $event['EventStartDate'] ) && $event['EventEndDate'] == $beginning_of_time ) {
				$event['EventEndDate'] = $event['EventStartDate'];
			}

			$out = array(
				'cb'         => $event['_uid'],
				'start_date' => date( $date_format, strtotime( $event['EventStartDate'] ) ),
				'end_date'   => date( $date_format, strtotime( $event['EventEndDate'] ) ),
				'event'      => $event['post_title'],
				);
			$table->items[] = $out;

		}

		$table->set_pagination_args( array(
			'total_items' => count( $possible_events ),
			'per_page'    => 30,
		) );

		ob_start();
			if ( self::is_search_for_recurring() ) {
				?>
				<div class="description">
					<span class="description">
						<?php esc_html_e( 'This is a preview of the type of content you will be getting in your recurring import.', 'tribe-events-ical-importer' ); ?>
					</span>
				</div>
				<?php
			}
			$table->display();

			tribe_ical_importer_get_template( 'status-category-selector' );

			if ( ! self::is_search_for_recurring() ) {
				tribe_ical_importer_get_template( 'one-time-buttons' );
			} else {
				tribe_ical_importer_get_template( 'recurring-buttons' );
			}

		return ob_get_clean();

	}

	/**
	 * is_search_for_recurring
	 *
	 * Did we click the Search button?
	 * Did we select a Recurring type before clicking the button?
	 *
	 * @static
	 *
	 * @return bool
	 */
	public static function is_search_for_recurring() {
		if ( empty( $_REQUEST[ 'schedule' ] ) ) {
			return false;
		}
		if ( $_REQUEST[ 'schedule' ] === '1' ) {
			return false;
		}
		return true;
	}

	/**
	 * Process Import Submission
	 *
	 * Process an import from either a scheduled import or a form submission
	 *
	 * @param array [$args] - things like schedule
	 * @param Tribe__Events__Ical_Importer__Recurring_Imports [$recurring] - used currently only for unit tests
	 *
	 * @return array( 'inserted' => int, 'updated' => int, 'scanned' => int ) The number of events that were imported.
	 *
	 */
	public function processImportSubmission( $args = array(), Tribe__Events__Ical_Importer__Recurring_Imports $recurring = null ) {
		$recurring = $recurring ? $recurring : Tribe__Events__Ical_Importer__Recurring_Imports::get_instance();
		$failed = false;
		$scheduled = false;
		if ( ! empty( $args[ 'key' ] ) ) {
			$this->saved_import_key = $args[ 'key'];
			$scheduled = true;
		}

		$this->setLimits();

		//let the world know we are running
		if ( $scheduled ) {
			$recurring->add_failed_recurring_import( $args[ 'key' ] );
		}

		//make sure we have the saved import before running cron
		if ( ! empty( $args[ 'schedule' ] ) ) {
			if ( ! $recurring->checkForSavedImport( $args ) ) {
				if ( $scheduled ) {
					$recurring->remove_failed_recurring_import( $args[ 'key'] );
				}

				return array(
					'error' => __( 'The Saved Import Does Not Exist', 'tribe-events-ical-importer' ),
				);
			}
		}


		// When an import happens during a cron task, $args will already be set: for manual imports
		// we need to base the args on the posted request
		if ( empty( $args ) ) {
			$args = $_POST;
			$args['manual'] = true;

			if ( ! empty( $args['start'] ) ) {
				$datepicker_format = Tribe__Date_Utils::datepicker_formats( tribe_get_option( 'datepickerFormat' ) );
				$args['start'] = Tribe__Date_Utils::datetime_from_format( $datepicker_format, $args['start'] );
			}
		}

		$events = $this->getEventsData( $args );

		//we errored out
		if ( ! $events instanceof WP_Error ) {
			if ( ! empty( $args[ 'events_to_import' ] ) ) {
				foreach ( $events as $k => $event ) {
					if ( ! in_array( $event[ '_uid' ], $args[ 'events_to_import' ] ) ) {
						unset( $events[ $k ] );
					}
				}
			}

			$num_imported_events = $this->saveEvents( $events, $args );

			if ( isset( $args[ 'schedule' ] ) && $args[ 'schedule' ] != '1' ) {
				$recurring->updateScheduleData( $args, $num_imported_events[ 'inserted' ] );
			}

			if ( ! empty( $args[ 'expected_count' ] ) ) {
				$count = $num_imported_events[ 'updated' ] + $num_imported_events[ 'inserted' ];
				if ( $count != $args[ 'expected_count' ] ) {
					$num_imported_events[ 'expected' ] = $args[ 'expected_count' ] - $count;
				}
			}
		} else {

			$failed = true;
			$num_imported_events  = array(
				'updated'  => 0,
				'inserted' => 0,
				'scanned'  => 0,
			);
		}

		//let the world know we are done
		if ( $scheduled && ! $failed ) {
			$recurring->remove_failed_recurring_import( $args[ 'key'] );
		}

		return $num_imported_events;

	}

	/**
	 * Dismiss Saved Imports Warning
	 *
	 * Clear out the saved imports running errors
	 *
	 * @uses $_REQUEST[ 'dismiss_saved_imports' ]
	 *
	 * @return void
	 */
	public function dismissSavedImportsWarning() {
		if ( ! empty( $_REQUEST[ 'dismiss_saved_imports' ] ) ) {
			Tribe__Events__Ical_Importer__Recurring_Imports::get_instance()->remove_failed_recurring_import( false );
		}

	}

	/**
	 * Get Events Data
	 *
	 * Retrieves events from either an ical or a ics file
	 *
	 *
	 * @return array
	 */
	protected function getEventsData( $args = false ) {

		if ( empty( $args ) ) {
			$args = $_POST;
		}

		if ( empty( $args['start'] ) ){
			$args['start'] = date( 'Y-m-d' );
		} else {
			$datepicker_format = Tribe__Date_Utils::datepicker_formats( tribe_get_option( 'datepickerFormat' ) );
			$args['start'] = Tribe__Date_Utils::datetime_from_format( $datepicker_format, $args['start'] );
		}

		$this->startDate = $args['start'];

		if ( empty( $args['end'] ) ){
			$args['end'] = date( 'Y-m-d', strtotime( '+2 year' ) );
		}
		$this->endDate = $args['end'];

		if ( ! empty( $args['location'] ) ) {
			$this->location = $args['location'];
		}

		if ( ! empty( $args['radius']  ) ) {
			$this->radius = $args['radius'];
		}

		if ( ! empty( $args['category'] ) ) {
			$this->category = $args['category'];
		}

		if ( ! empty( $args['keywords'] ) ) {
			$keywords = str_replace( ',', ' ', $args['keywords'] );
			$args[ 'keywords' ] = explode( ' ', $keywords );
			$this->keywords = $args['keywords'];
		}

		if ( ! empty( $args['url'] ) ) {
			$ical = $args['url'];
		} else {
			$ical = $args['file'];
		}

		if ( ! empty( $args['events_to_import'] ) ) {
			$args['manual'] = true;
		}

		unset( $args[ 'expected' ] );

		$ev = new Tribe__Events__Ical_Importer__Feed_Parser( $ical, $args );
		$events = $ev->get_events();

		self::$ical_source = $ev->source;

		if ( empty( $events ) ) {
			//if all events were filtered out
			if ( ! empty( $ev->raw_events ) ) {
				return $this->getSearchError( $ev->matched_search_criteria );
			}
		}

		return $ev->get_events();
	}

	/**
	 * Get Search Error
	 *
	 * Returns appropriate error for an empty search based on matches as search criteria
	 *
	 * @param array $matching_search_criteria
	 *
	 * @return WP_Error
	 */
	private function getSearchError( $matched_search_criteria ) {

		$failed = array();

		foreach ( $matched_search_criteria as $type => $result ) {
			if ( $this->{$type} != null ) {
				if ( ! $result ) {
					$failed[] = $type;
				}
			}
		}

		//generate error for failed
		switch ( $failed ) {
			case array( 'startDate' ):
				return new WP_Error( 1, __( 'Your Start Date returned no results. Please adjust your Start Date.', 'tribe-events-ical-importer' ) );
			break;

			case array( 'category' ):
				return new WP_Error( 1, __( 'Your Category returned no results. Please adjust your Category.', 'tribe-events-ical-importer' ) );
			break;

			case array( 'keywords' ):
				return new WP_Error( 1, __( 'Your Keywords returned no results. Please adjust your Keywords.', 'tribe-events-ical-importer' ) );
			break;

			case array( 'location' ):
				return new WP_Error( 1, __( 'Your Location returned no results. Please select a broader radius.', 'tribe-events-ical-importer' ) );
			break;

			//more than one item so we need a generic message
			default:
				return new WP_Error( 1, __( 'Your search returned no results. Please adjust your search.', 'tribe-events-ical-importer' ) );
			break;

		}

	}

	/**
	 * Generate the import page.
	 *
	 *
	 *
	 *
	 * @return void
	 */
	public function doImportPage() {
		$possible_events_list = $this->parseIntoEventsList( $this->possibleEvents );

		include( trailingslashit( $this->pluginPath ) . 'src/admin-views/import-page.php' );
	}

	/**
	 * get_radii
	 *
	 * Get the possible radius value by miles
	 * or kilometers based on settings
	 *
	 * @static
	 *
	 * @return array|mixed|void
	 */
	public static function get_radii() {
		$unit = Tribe__Events__Importer__Options::getOption( 'geoloc_default_unit', 'miles' );

		if ( $unit == 'miles' ) {
			$mi_string = _x( '%1$s (mi)', 'X (miles abbreviation)', 'tribe-events-ical-importer' );
			$radii = array(
				sprintf( $mi_string, 1 ) => 1.6,
				sprintf( $mi_string, 5 ) => 8,
				sprintf( $mi_string, 10 ) => 16.1,
				sprintf( $mi_string, 25 ) => 40.2,
				sprintf( $mi_string, 50 ) => 80.5,
				sprintf( $mi_string, 100 ) => 160.9,
			);
		} else {
			$km_string = _x( '%1$s (km)', 'X (kilometers abbreviation)', 'tribe-events-ical-importer' );
			$radii = array(
				sprintf( $km_string, 1 ) => 1,
				sprintf( $km_string, 5 ) => 5,
				sprintf( $km_string, 10 ) => 10,
				sprintf( $km_string, 25 ) => 25,
				sprintf( $km_string, 50 ) => 50,
				sprintf( $km_string, 100 ) => 100,
			);
		}
		$radii = apply_filters( 'tribe-events-importer-ical-radii', $radii );

		return $radii;
	}

	public static function getRadiusAbbreviation() {
		$unit = Tribe__Events__Importer__Options::getOption( 'geoloc_default_unit', 'miles' );

		if ( $unit == 'miles' ) {
			return _x( 'mi', 'Abbreviation for the miles unit of measure', 'tribe-events-ical-importer' );
		} else {
			return _x( 'km', 'Abbreviation for the kilometers unit of measure', 'tribe-events-ical-importer' );
		}
	}

	/**
	 * Km to Miles Maybe Convert
	 *
	 * Quick an easy converting km to miles.
	 * All the default caculations are done with km so this if for outputs in miles etc
	 *
	 * Will return the proper display string based on the settings
	 *
	 * @param float $km
	 *
	 * @return string
	 *
	 */
	public static function kmToMilesMaybeConvert( $km ) {
		$unit = Tribe__Events__Importer__Options::getOption( 'geoloc_default_unit', 'miles' );

		if ( $unit != 'miles' ) {
			return $km .' KM';
		}

		$possibles = array(
			1    => 1.6,
			5    => 8,
			10   => 16.1,
			25   => 40.2,
			100  => 160.9,
		);


		$miles = array_search( $km, $possibles );

		if ( ! empty( $miles ) ) {
			 return $miles;
		}

		return ceil( $km * 0.621371 ). ' Mi';
	}

	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::$instance = self::instance();
	}

	/**
	 * Get (and instantiate, if necessary) the instance of the class
	 *
	 * @static
	 * @return Tribe__Events__Ical_Importer__Main
	 */
	public static function instance() {
		if ( ! self::$instance instanceof self ) {
			self::$instance = new self();
		}
		return self::$instance;
	}
}
