<?php
/**
 * Tribe__Events__Ical_Importer__Recurring_Imports
 *
 * Manage the Saved Recurring Imports
 *
 * @example Tribe__Events__Ical_Importer__Recurring_Imports::get_instance()->get_saved_imports();
 *
 */
class Tribe__Events__Ical_Importer__Recurring_Imports {

	const FAILED_RECURRING = 'tribe_events_ical-importer_running_processes';
	const SAVED_IMPORTS = 'tribe-events-importexport-ical-importer-saved-imports';

	const CRON = 'tribe_events_ical-importer_do_scheduled_import';

	const NONCE = 'tribe-import-edit-save-recurring-form';


	private function __construct() {
		$this->hooks();
	}


	private function hooks() {
		add_action( self::CRON, array( Tribe__Events__Ical_Importer__Main::instance(), 'processImportSubmission' ), 10, 1 );
		add_action( 'init', array( $this, 'maybe_save_import_update' ) );
	}


	/**
	 * maybe_save_import_update
	 *
	 * If the edit form is submitted update the recurring import
	 *
	 * @return void
	 */
	public function maybe_save_import_update() {
		if ( ! empty( $_POST[ self::NONCE ] ) && wp_verify_nonce( $_POST[ self::NONCE ], self::NONCE ) ) {
			$defaults = array(
				'location'            => null,
				'radius'              => null,
				'start'               => null,
				'keywords'            => null,
				'import_category'     => null,
				'recurring_overwrite' => '',
			);
			$args = wp_parse_args( $_POST, $defaults );

			$import_query = array(
				'location'            => trim( strip_tags( $args[ 'location' ] ) ),
				'radius'              => $args[ 'radius' ],
				'import_category'     => trim( strip_tags( $args[ 'import_category' ] ) ),
				'schedule'            => $args[ 'schedule' ],
				'keywords'            => trim( strip_tags( $args[ 'keywords' ] ) ),
				'post_status'         => $args[ 'post_status' ],
				'recurring_overwrite' => ! empty( $args[ 'recurring_overwrite' ] ) ? 'checked' : '',
			);
			$start = strtotime( $args[ 'start' ] );
			if ( $start ) {
				$import_query[ 'start' ] = date( 'm/d/y', $start );
			}

			$saved_queries = $this->get_saved_imports();

			$query = $saved_queries[ $args[ 'key' ] ];
			$import_query = wp_parse_args( $import_query, $query );
			$saved_queries[ $args[ 'key' ] ] = $import_query;

			//args will change so have to delete old scheduled
			$this->deleteSavedImportQuery( $args[ 'key' ] );

			$this->update_saved_imports( $saved_queries );

			$this->scheduleImportQuery( $import_query );

			wp_redirect( add_query_arg( array( 'edit_recurring' => 1 ), Tribe__Events__Ical_Importer__Main::instance()->get_url() ) );
			die;
		}
	}


	public function get_saved_imports() {
		$defaults = array(
			'location'            => null,
			'radius'              => null,
			'start'               => null,
			'keywords'            => null,
			'import_category'     => null,
			'recurring_overwrite' => '',
		);

		$imports = get_option( self::SAVED_IMPORTS, array() );
		$keyed   = array();
		foreach ( $imports as $_import ) {
			$_import = wp_parse_args( $_import, $defaults );

			$keyed[ $_import[ 'key' ] ] = $_import;
		}

		return $keyed;
	}


	public function update_saved_imports( $saved_imports ) {
		return update_option( self::SAVED_IMPORTS, $saved_imports );
	}


	/**
	 * Clear saved imports
	 *
	 * Removes cron jobs and saved imports
	 * Used when plugin is deactivated
	 *
	 * @return void
	 */
	public function clear_saved_imports() {
		$saved_imports = $this->get_saved_imports();
		foreach ( $saved_imports as $saved_import ) {
			unset( $saved_import[ 'last_update' ] );
			unset( $saved_import[ 'number_imported' ] );
			wp_clear_scheduled_hook( self::CRON, array( $saved_import ) );
		}

		wp_clear_scheduled_hook( self::CRON );

		$this->remove_failed_recurring_import();
		$this->update_saved_imports( array() );
	}


	/**
	 * Delete Saved Import Query
	 *
	 * Delete a saved import query by index.
	 *
	 * @param int $index the array index of the saved query.
	 *
	 * @return bool Whether the deletion was successful.
	 *
	 */
	public function deleteSavedImportQuery( $index = null ) {
		$success = false;

		if ( null !== $index ){

			$saved_imports = $this->get_saved_imports();

			$_import = $saved_imports[ $index ];
			//because this will change upon each run
			unset( $_import[ 'last_update' ] );
			unset( $_import[ 'number_imported' ] );

			wp_clear_scheduled_hook( self::CRON, array( $_import ) );

			unset( $saved_imports[ $index ] );
			$success = $this->update_saved_imports( $saved_imports );

			$this->remove_failed_recurring_import( $_import[ 'key' ] );
		}

		return $success;
	}


	/**
	 * Run Saved Import Query
	 *
	 * Run a saved import query.
	 *
	 * The response will either be a breakdown of event counts as follows:
	 *
	 *     [ 'inserted' => int, 'updated' => int, 'scanned' => int ]
	 *
	 * Or else an error message:
	 *
	 *     [ 'error' => string ]
	 *
	 * @param int $index the array index of the saved query.
	 *
	 * @return array
	 */
	public function runSavedImportQuery( $index ) {
		if ( isset( $index ) ) {
			$saved_imports = $this->get_saved_imports();

			$args = $saved_imports[ $index ];

			if ( ! empty( $saved_imports[ $index ][ 'key' ] ) ) {
				$this->saved_import_key = $saved_imports[ $index ][ 'key' ];
			}

			$count = Tribe__Events__Ical_Importer__Main::instance()->processImportSubmission( $args );

		} else {
			return array(
				'error' => __( 'The Saved Import Does Not Exist', 'tribe-events-ical-importer' ),
			);
		}

		return $count;
	}


	/**
	 * Schedule the import query.
	 *
	 * Schedule a wp-cron event to run an import
	 *
	 * @param array $args - The args to pass into the action hook. Should be the same as the saved array for the scheduled import.
	 *
	 * @return bool Whether the scheduling was successful.
	 */
	public function scheduleImportQuery( $args ) {
		//because this will change upon each run
		unset( $args[ 'last_update' ] );
		unset( $args[ 'number_imported' ] );

		$scheduled = wp_schedule_event( time(), $args[ 'schedule' ], self::CRON, array( $args ) );

		if ( $scheduled !== false ) {
			return true;
		}

		return false;
	}


	/**
	 * Save Import Query
	 *
	 * Save an import query.
	 *
	 * @return bool - true on success
	 */
	public function saveImportQuery() {
		$defaults = array(
			'location'            => null,
			'radius'              => null,
			'start'               => null,
			'keywords'            => null,
			'import_category'     => null,
			'post_status'         => $status = Tribe__Events__Importer__Options::get_default_post_status( 'ical' ),
			'recurring_overwrite' => '',
		);

		$args = wp_parse_args( $_POST, $defaults );

		$domain = @parse_url( trim( $args[ 'url' ] ) );

		$import_query = array(
			'domain'              => ! empty( $domain[ 'host' ] ) ? $domain[ 'host' ] : '',
			'key'                 => time(),
			'url'                 => trim( strip_tags( $args[ 'url' ] ) ),
			'location'            => trim( strip_tags( $args[ 'location' ] ) ),
			'radius'              => $args[ 'radius' ],
			'import_category'     => trim( strip_tags( $args[ 'import_category' ] ) ),
			'start'               => $args[ 'start' ],
			'schedule'            => $args[ 'schedule' ],
			'initiated'           => date( 'm/d/Y' ),
			'keywords'            => trim( strip_tags( $args[ 'keywords' ] ) ),
			'post_status'         => $args[ 'post_status' ],
			'recurring_overwrite' => ! empty( $args[ 'recurring_overwrite' ] ) ? 'checked' : '',
			'last_update'         => '',
			'number_imported'     => '1',
		);

		$saved_queries = $this->get_saved_imports();

		$saved_queries[ ] = $import_query;

		$this->update_saved_imports( $saved_queries );

		return $import_query;
	}


	/**
	 * Check for Saved Import
	 *
	 * Verify an import exists
	 *
	 * @param array $args
	 *
	 * @return bool
	 */
	public function checkForSavedImport( $args ) {
		$saved_queries = $this->get_saved_imports();
		if ( in_array( $args[ 'key' ], wp_list_pluck( $saved_queries, 'key' ) ) ) {
			return true;
		}

		return false;

	}


	/**
	 * Update Schedule Data
	 *
	 * Saves the latest data to schedules import which just ran
	 *
	 * @uses called at the end of self::processImportSubmission() only when required
	 *
	 * @param array $args
	 * @param int   $num_imported_events
	 *
	 * @return void
	 *
	 */
	public function updateScheduleData( $args, $num_imported_events = 0 ) {

		$saved_imports = $this->get_saved_imports();

		$key = false;
		foreach ( $saved_imports as $k => $_import ) {
			if ( $_import[ 'key' ] == $args[ 'key' ] ) {
				$key = $k;
				break;
			}
		}

		if ( $key !== false ) {
			$saved_imports[ $key ][ 'number_imported' ] = (int) $saved_imports[ $key ][ 'number_imported' ] + $num_imported_events;
			$saved_imports[ $key ][ 'last_update' ]     = date_i18n( 'm/d/Y h:i' );

			$this->update_saved_imports( $saved_imports );
		}
	}


	/**
	 * Get Failed Saved Import
	 *
	 * Gets the keys of any Saved Imports which did not complete
	 *
	 * @uses if a key is specified will return true if did not complete and false if did
	 *
	 * @param float $key - if we want to check for a certain one
	 *
	 * @return array|bool
	 *
	 */
	public function get_failed_recurring_imports( $key = false ) {
		$running = get_option( self::FAILED_RECURRING, array() );
		if ( $key ){
			foreach ( $running as $saved ) {
				if ( (float) trim( $saved ) == (float) trim( $key ) ) {
					return true;
				}
			}
			return false;
		}

		return $running;
	}


	/**
	 * Add Failed Recurring Import
	 *
	 * Add a Saved Imports key to running saved imports
	 *
	 * @return void
	 */
	public function add_failed_recurring_import( $key ) {
		$running = $this->get_failed_recurring_imports();

		if ( ! in_array( $key, $running ) ) {
			$running[] = $key;
		}

		update_option( self::FAILED_RECURRING, $running );
	}

	/**
	 * update_failed_recurring_imports
	 *
	 * @param array $failed
	 *
	 * @internal for unit testing
	 *
	 * @return void
	 */
	public function update_failed_recurring_imports( $failed ) {
		update_option( self::FAILED_RECURRING, $failed );
	}


	/**
	 * Remove Failed Recurring Import
	 *
	 * Remove a Saved Imports key to running saved imports
	 *
	 * @param float|bool $key - false will remove all
	 *
	 * @return void
	 */
	public function remove_failed_recurring_import( $key = false ) {
		$running = $this->get_failed_recurring_imports();

		if ( $key ) {
			foreach ( $running as $k => $saved ) {
				if ( (float) trim( $saved ) == (float) trim( $key ) ) {
					unset( $running[ $k ] );
				}
			}
		} else {
			$running = array();
		}

		update_option( self::FAILED_RECURRING, $running );
	}

	//********** SINGLETON FUNCTIONS **********/

	/**
	 * Instance of this class for use as singleton
	 */
	private static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::$instance = self::get_instance();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return Tribe__Events__Ical_Importer__Recurring_Imports
	 */
	public static function get_instance() {
		if ( ! self::$instance instanceof self ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}
