<?php
/**
 * Controls the registration of different importers.
 */

// Don't load directly
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

if ( class_exists( 'Tribe__Events__Importer__Registrar' ) ) {
	return;
}

class Tribe__Events__Importer__Registrar {
	/**
	 * @static
	 * @var Tribe__Events__Importer__Registrar The instance of the class.
	 */
	protected static $instance;

	/**
	 * @var array The importers.
	 */
	protected $import_apis;

	/**
	 * @var array The exporters.
	 */
	protected $export_apis;

	/**
	 * @var array The mu default options.
	 */
	protected $importExportDefaultMuOptions;



	protected function __construct() {
		$this->errors = array();
		$this->messages = array();
		$this->options = array();

		// Load multisite defaults
		if ( is_multisite() ) {
			$tribe_events_importexport_mu_default_options = array();
			if ( file_exists( WP_CONTENT_DIR . '/tribe-events-mu-defaults.php' ) )
				include( WP_CONTENT_DIR . '/tribe-events-mu-defaults.php' );
			$this->importExportDefaultMuOptions = apply_filters( 'tribe_mu_events_importexport_default_options', $tribe_events_importexport_mu_default_options );
		}

		$this->addActions();
		$this->addFilters();
	}



	/**
	 * Add actions.
	 */
	protected function addActions() {
		add_action( 'admin_notices', array( $this, 'displayMessages' ) );
		add_action( 'admin_notices', array( $this, 'displayErrors' ) );
	}

	/**
	 * Add filters.
	 */
	protected function addFilters() {
		add_filter( 'cron_schedules', array( $this, 'addCronIntervals' ) );
	}

	/**
	 * Display notification messages.
	 */
	public function displayMessages() {
		if ( isset( $this->messages ) && is_array( $this->messages ) ) {
			foreach ( $this->messages as $message ) {
				echo '<div class="updated">';
				echo '<p>' . $message . '</p>';
				echo '</div>';
			}
		}
	}

	/**
	 * Display notification messages.
	 */
	public function displayErrors() {
		if ( isset( $this->errors ) && is_array( $this->errors ) ) {
			foreach ( $this->errors as $error ) {
				echo '<div class="error">';
				echo '<p>' . $error . '</p>';
				echo '</div>';
			}
		}
	}


	/**
	 * Draw the import submenu page.
	 */
	public function doImportExportPage() {
		$this->export_apis = apply_filters( 'tribe-events-importexport-export-apis', array() );
		$this->import_apis = apply_filters( 'tribe-events-importexport-import-apis', array() );

		do_action( 'tribe_events_importexport_top' );

		?>
		<div class="tribe_events_importexport wrap">
			<div class="wrap">
				<h2><?php printf( $this->menuPageTitle ); ?></h2>
				<?php

				do_action( 'tribe_events_importexport_above_tabs' );

				$this->generateTabs( $this->currentTab );

				do_action( 'tribe_events_importexport_below_tabs' );
				do_action( 'tribe_events_importexport_below_tabs_tab_' . $this->currentTab );
				?>
				<div class="tribe-events_importexport-content">
					<?php
					do_action( 'tribe_events_importexport_before_content' );
					do_action( 'tribe_events_importexport_before_content_tab_' . $this->currentTab );

					do_action( 'tribe_events_importexport_content_tab_' . $this->currentTab );

					do_action( 'tribe_events_importexport_after_content_tab_' . $this->currentTab );
					do_action( 'tribe_events_importexport_after_content' );
					?>
				</div>

				<?php
				do_action( 'tribe_events_importexport_after_form_div' );
				?>
			</div>
		</div>
		<?php

		do_action( 'tribe_events_importexport_bottom' );

	}

	/**
	 * Generate the Import Tabs
	 */
	protected function generateTabs() {
		?>
		<h2 id="tribe-events-importexport-tabs" class="nav-tab-wrapper">
			<?php
			$tab   = 'general';
			$name  = apply_filters( 'tribe-events-importexport-general-tab-title', __( 'General', 'tribe-events-ical-importer' ) );
			$class = ( $tab == $this->currentTab ) ? ' nav-tab-active' : '';

			echo '<a id="' . esc_attr( $tab ) . '" class="nav-tab' . esc_attr( $class ) . '" href="' . esc_url( '?post_type=' . Tribe__Events__Main::POSTTYPE . '&page=' . $this->slug . '&tab=' . urlencode( $tab ) ) . '">' . $name . '</a>';

			if ( is_array( $this->import_apis ) && ! empty( $this->import_apis ) ) {
				foreach ( $this->import_apis as $importer ) {
					$tab   = esc_attr( $importer[ 'slug' ] );
					$name  = esc_attr( $importer[ 'name' ] );
					$class = ( $tab == $this->currentTab ) ? ' nav-tab-active' : '';
					echo '<a id="' . esc_attr( $tab ) . '" class="nav-tab' . esc_attr( $class ) . '" href="' . esc_url( '?post_type=' . Tribe__Events__Main::POSTTYPE . '&page=' . $this->slug . '&tab=' . urlencode( $tab ) ) . '">' . esc_html__( 'Import: ', 'tribe-events-ical-importer' ) . $name . '</a>';

				}

				do_action( 'tribe_events_importexport_after_tabs' );

			}
			?>
		</h2>
		<?php
	}


	/**
	 * Add weekly and monthly cron schedules.
	 */
	public function addCronIntervals() {
		$schedules['weekly'] = array(
			'interval' => 604800,
			'display' => __( 'Once Weekly', 'tribe-events-ical-importer' ),
		);
		$schedules['monthly'] = array(
			'interval' => 2635200,
			'display' => __( 'Once Monthly', 'tribe-events-ical-importer' ),
		);
		return $schedules;
	}


	/**
	 * Create the plugin instance and include the other class.
	 */
	public static function init() {
		if ( ! isset( self::$instance ) ) {
			self::$instance = self::instance();
		}
	}

	/**
	 * The singleton function.
	 *
	 * @return Tribe__Events__Importer__Registrar The instance.
	 */
	public static function instance() {
		if ( ! self::$instance instanceof self ) {
			self::$instance = new self();
		}
		return self::$instance;
	}
}
