<?php
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

if ( class_exists( 'Tribe__Events__Ical_Importer__Submission_Scrubber' ) ) {
	return;
}

/**
 * Tribe Events iCal Importer Submission Scrubber
 *
 * Removes all inappropriate date from a retrieved event
 *
 * @package Tribe__Events__Ical_Importer__Main
 *
 */
class Tribe__Events__Ical_Importer__Submission_Scrubber {

	/**
	 * Submission
	 *
	 * The event
	 *
	 * @var array
	 */
	protected $submission = array();

	/**
	 * Filters
	 *
	 * The filters to run against content type fields
	 *
	 * @var array
	 */
	protected $filters = array(
		'wp_kses_post',
		'strip_shortcodes',
	);


	/**
	 * Allowed Fields
	 *
	 * Top level fields allowed
	 *
	 * @var array
	 */
	protected $allowed_fields = array(
		'ID',
		'post_content',
		'post_title',
		'EventAllDay',
		'EventStartDate',
		'EventStartHour',
		'EventStartMinute',
		'EventStartMeridian',
		'EventEndDate',
		'EventEndHour',
		'EventEndMinute',
		'EventEndMeridian',
		'EventShowMapLink',
		'EventShowMap',
		'EventURL',
		'EventCurrencySymbol',
		'EventCost',
		'Venue',
		'Organizer',
		'is_recurring',
		'recurrence',
		'_uid',
		'parent_uid',
		'categories',
		'EventTimezone',
	);

	/**
	 * Allowed Venue Fields
	 *
	 * Fields a venue may have
	 *
	 * @var array
	 */
	protected $allowed_venue_fields = array(
		'VenueID',
		'Venue',
		'Address',
		'City',
		'Country',
		'Province',
		'State',
		'Zip',
		'Phone',
		'OverwriteCoords',
		'Lat',
		'Lng',
		'ShowMap',
		'ShowMapLink',
		'EventShowMap',
		'EventShowMapLink',
	);

	/**
	 * Allowed Organizer Fields
	 *
	 * Fields organizers are allowed to have
	 *
	 * @var array
	 */
	protected $allowed_organizer_fields = array(
		'OrganizerID',
		'Organizer',
		'Phone',
		'Website',
		'Email',
	);

	/**
	 * Constructor
	 *
	 *
	 */
	public function __construct( array $submission ) {
		$this->submission = $submission;
	}


	/**
	 * Remove data from the submission that shouldn't be there
	 *
	 * @return array The cleaned submission
	 */
	public function scrub() {
		add_filter( 'wp_kses_allowed_html', array( $this, 'filter_allowed_html_tags' ), 10 );

		if ( ! empty( $this->submission['Venue'] ) ) {
			$this->set_venue();
		}

		if ( ! empty( $this->submission['Organizer'] ) ) {
			$this->set_organizer();
		}

		$this->remove_unexpected_fields();
		$this->filter_field_contents();

		remove_filter( 'wp_kses_allowed_html', array( $this, 'filter_allowed_html_tags' ), 10 );

		return $this->submission;
	}

    /**
	 * Filter Allowed Html Tags
	 *
	 * Remove some tags from the allowed list
	 *
	 * @param array $tags
	 *
	 * @return array
	 */
	public function filter_allowed_html_tags( $tags ) {
		unset( $tags['form'] );
		unset( $tags['button'] );
		return $tags;
	}


    /**
	 * Set Venue
	 *
	 * Scrub the venue data
	 *
	 * @return void
	 */
	protected function set_venue() {
		$this->submission['Venue'] = stripslashes_deep( $this->submission['Venue'] );
		$this->submission['Venue'] = $this->filter_venue_data( $this->submission['Venue'] );

	}


	/**
	 * Filter Venue Data
	 *
	 * Parse each venue field one by one
	 *
	 * @var array $venue_data
	 *
	 * @return array
	 */
	protected function filter_venue_data( $venue_data ) {
		$fields = array(
			'Venue',
			'Address',
			'City',
			'Country',
			'Province',
			'State',
			'Zip',
			'Phone',
		);

		foreach ( $fields as $field ) {
			if ( isset( $venue_data[ $field ] ) ) {
				$venue_data[ $field ] = $this->filter_string( $venue_data[ $field ] );
			}
		}

		return $venue_data;
	}


	 /**
	 * Set Organizer
	 *
	 * Scrub the organizer data
	 *
	 * @return void
	 */
	protected function set_organizer() {
		$this->submission['Organizer'] = stripslashes_deep( $this->submission['Organizer'] );
		$this->submission['Organizer'] = $this->filter_organizer_data( $this->submission['Organizer'] );
	}


    /**
	 * Filter Organizer Data
	 *
	 * Parse each organizer field one by one
	 *
	 * @var array $organizer_data
	 *
	 * @return array
	 */
	protected function filter_organizer_data( $organizer_data ) {
		$fields = array(
			'Organizer',
			'Phone',
			'Website',
			'Email',
		);

		foreach ( $fields as $field ) {
			if ( isset( $organizer_data[ $field ] ) ) {
				$organizer_data[ $field ] = $this->filter_string( $organizer_data[ $field ] );
			}
		}
		return $organizer_data;
	}


	/**
	 * Remove Unexpected Fields
	 *
	 * Remove any fields that don't belong
	 *
	 * @return void
	 */
	protected function remove_unexpected_fields() {
		$allowed_fields = $this->get_allowed_event_fields();

		foreach ( $this->submission as $key => $value ) {
			if ( ! in_array( $key, $allowed_fields ) ) {
				unset( $this->submission[ $key ] );
			}
		}

		if ( ! empty( $this->submission['Venue'] ) ) {
			$allowed_venue_fields = $this->allowed_venue_fields;
			foreach ( $this->submission['Venue'] as $key => $value ) {
				if ( ! in_array( $key, $allowed_venue_fields ) ) {
					unset( $this->submission['Venue'][ $key ] );
				}
			}
		}

		if ( ! empty( $this->submission['Organizer'] ) ) {
			$allowed_organizer_fields = $this->allowed_organizer_fields;
			foreach ( $this->submission['Organizer'] as $key => $value ) {
				if ( ! in_array( $key, $allowed_organizer_fields ) ) {
					unset( $this->submission['Organizer'][ $key ] );
				}
			}
		}
	}


	/**
	 * Get Allowed Event Fields
	 *
	 * Merge $this->allowed_fields with tribe_get_option('custom-fields')
	 *
	 * @return array
	 */
	protected function get_allowed_event_fields() {

		return array_merge( $this->allowed_fields, $this->get_custom_field_keys() );

	}


	/**
	 * Get Custom Field Keys
	 *
	 * Get the custom fields tribe_get_option('custom-fields')
	 *
	 * @return array
	 */
	protected function get_custom_field_keys() {
		$customFields = tribe_get_option( 'custom-fields' );
		if ( empty( $customFields ) || ! is_array( $customFields ) ) {
			return array();
		}
		$keys = array();
		foreach ( $customFields as $field ) {
			$keys[] = $field['name'];
		}
		return $keys;
	}


	/**
	 * Filter Field Contents
	 *
	 * Run $this->filter_string over the content type fields
	 *
	 * @return void
	 */
	protected function filter_field_contents() {
		$fields = array(
			'post_content',
			'post_title',
			'EventURL',
			'EventCurrencySymbol',
			'EventCost',
		);

		foreach ( $fields as $field ) {
			if ( isset( $this->submission[ $field ] ) ) {
				$this->submission[ $field ] = $this->filter_string( $this->submission[ $field ] );
			}
		}
	}

	/**
	 * Filter String
	 *
	 * Run filters specified by $this->filters against a string
	 *
	 * @param string $string
	 *
	 * @return string
	 */
	protected function filter_string( $string ) {
		foreach ( $this->filters as $callback ) {
			$string = call_user_func( $callback, $string );
		}
		return $string;
	}
}
